unit main;

interface
uses ucimport,cnvtypes,sysutils;

function cnvEnumFormats(nFormatId:integer;pcFormatName:PChar;var BufSize:integer;var Flags:integer):integer;stdcall;
function cnvImport(nFormatId:integer;pcFileName:PChar;ASpec:hSpec):integer;stdcall;
function cnvExport(nFormatId:integer;pcFileName:PChar;ASpec:hSpec):integer;stdcall;

implementation

const
  fASCII = 1;
  fBinary = 2;

  sFormats:array[fAscii..fBinary] of string=
      ('ASCII Text (*.ttt)|*.ttt',
       'Custom Full Binary Format (*.cool)|*.cool');

function cnvEnumFormats(nFormatId:integer;pcFormatName:PChar;var BufSize:integer;var Flags:integer):integer;
begin
  Result:=0;
  case nFormatId of
  0: Result:= 2; // we support 2 formats
  fAscii,fBinary:
      begin
        StrLCopy(pcFormatName,PChar(sFormats[nFormatId]),BufSize-1);
        BufSize:=length(sFormats[nFormatId]);
        Flags:=FLAGS_READWRITE;
        Result:=1;
      end;
  end;
end;


// imports
function ImportBIN(pcFileName:PChar;ASpec:hSpec):integer;
var h:integer;
begin
  Result:=0;

  h:=FileOpen(pcFileName,fmOpenRead or fmShareDenyWrite);
  if h=-1 then exit;

  try
    Result:=ord(LoadFromStreamHandle(h,ASpec));
  finally
    FileClose(h);
  end;

end;

function ImportText(pcFileName:PChar;ASpec:hSpec):integer;
var f:text;
    allocpoints,npoints,nlay,ncol,i:integer;
    d:array of double;
begin
  Result:=0;
  {$I-}
  assign(f,pcFileName);
  reset(f);
  if IOResult<>0 then exit;

  npoints:=0;

  SetPropValue(ASpec,-1,'Lock',1);

  // Assume data is sampled at 10 Hz
  SetPropValue(ASpec,-1,'Freq',10);

  SetPropValue(ASpec,-1,'NPoints',100);

  allocpoints:=100;

  SetPropValue(ASpec,-1,'Lock',0);

  nlay:=0;

  while not eof(f) do
  begin
    ncol:=0;
    while not eoln(f) do
    begin
      if (length(d)<ncol+1) then
      begin
        inc(nlay);
        setlength(d,nlay);
        SetPropValue(ASpec,-1,'Layers',nlay);
      end;
      read(f,d[ncol]);
      inc(ncol);
    end;
    readln(f);
    inc(npoints);
    if npoints>=allocpoints then
    begin
      SetPropValue(ASpec,-1,'NPoints',npoints+1000);
      allocpoints:=npoints+1000;
    end;

    for i:=0 to nlay-1 do
    SetData(ASpec,i+1,npoints,d[i]);
  end;


  SetPropValue(ASpec,-1,'XEnd',(npoints-1)*(1/600));


  close(f);

  Result:=1;
end;

function cnvImport(nFormatId:integer;pcFileName:PChar;ASpec:hSpec):integer;
begin
  Result:=0;
  case nFormatId of
  fAscii:Result:=ImportText(pcFileName,ASpec);
  fBinary:Result:=ImportBIN(pcFileName,ASpec);
  end;
end;

// exports
function ExportBIN(pcFileName:PChar;ASpec:hSpec):integer;
var h:integer;
begin
  Result:=0;

  h:=FileCreate(pcFileName);

  if h=-1 then exit;

  try
    Result:=ord(StoreToStreamHandle(h,ASpec));
  finally
    FileClose(h);
  end;

end;

function ExportText(pcFileName:PChar;ASpec:hSpec):integer;
var f:text;
    npoints,nlay,i,j:integer;
    d:array of double;
    v:variant;
begin
  Result:=0;
  {$I-}
  assign(f,pcFileName);
  rewrite(f);
  if IOResult<>0 then exit;

  if GetPropValue(ASpec,-1,'NPoints',v) then
    npoints:=v
  else
    exit;

  if GetPropValue(ASpec,-1,'Layers',v) then
    nlay:=v
  else
    exit;

  for i:=0 to npoints-1 do
  begin
    for j:=0 to nlay-1 do
    begin
      write(f,FloatToStrF(GetData(ASpec,j+1,i+1),ffGeneral,9,5));
      if j<nlay-1 then write(f,#9);
    end;
    writeln(f);
  end;

  close(f);

  Result:=1;
end;



function cnvExport(nFormatId:integer;pcFileName:PChar;ASpec:hSpec):integer;
begin
  Result:=0;
  case nFormatId of
  fAscii:Result:=ExportText(pcFileName,ASpec);
  fBinary:Result:=ExportBin(pcFileName,ASpec);
  end;
end;


end.
