unit baseconv;

interface

{$ifdef FPC}
{$mode delphi}
{$endif}

uses
  Classes, SysUtils,cnvtypes,ucintf;


type
  TBaseConv=class
  protected
    class function FormatName:utf8string;virtual;abstract;
    class function FormatFlags:integer;virtual;abstract;
    function Imp(hSp:hSPEC;const Fname:PChar):integer;virtual;abstract;
    function Exp(hSp:hSPEC;const Fname:PChar):integer;virtual;abstract;
  public

  end;

  TBaseConvClass=class of TBaseConv;

  { TConvFactory }

  TConvFactory=class
  private
    fList:TList;
  public
    constructor Create;
    destructor Destroy;override;
    procedure Register(const ref:TbaseConvClass);overload;
    procedure Register(const ref:array of TbaseConvClass);overload;
    function FormatCount:integer;
    function FormatName(ndx:integer):utf8string;
    function FormatFlags(ndx:integer):integer;
    function CreateConv(nFormat:integer):TBaseConv;
  end;


function cnvEnumFormats(nFormat:integer;PBuf:PAnsiChar;var bufsize:integer;var flags:integer):integer;stdcall;
function cnvImport(nFormat:integer;const FName:PAnsiChar;hSp:HSPEC):integer;stdcall;
function cnvExport(nFormat:integer;const FName:PAnsiChar;hSp:HSPEC):integer;stdcall;
function CnvFact:TConvFactory;

implementation
const
  ffact:TConvFactory=nil;

function CnvFact: TConvFactory;
begin
  if not Assigned(ffact) then
     ffact:=TConvFactory.Create;
  Result:=ffact;
end;


function cnvEnumFormats(nFormat: integer; PBuf: PAnsiChar;var bufsize:integer;var flags:integer): integer;
var
  cnt:integer;
  bs:utf8string;
begin
  result:=CNV_FAIL;
  cnt:=CnvFact.FormatCount;
  if nFormat=0 then
     Result:=cnt // return number of formats
  else
  if (nFormat>0) and (nFormat<=cnt) then
  begin
    bs:=CnvFact.FormatName(nFormat)+#0;
    bufsize:=length(bs);
    move(bs[1],PBuf^,bufsize);
    flags:=CnvFact.FormatFlags(nFormat);
    Result:=CNV_OK;
  end;
end;

function cnvImport(nFormat:integer;const FName:PAnsiChar;hSp:HSPEC):integer;
var
  cnv:TBaseConv;
begin
  Result:=CNV_FAIL;
  cnv:=CnvFact.CreateConv(nFormat);
  try
    Result:=cnv.Imp(hSp,FName);
  finally
    cnv.Free;
  end;
end;

function cnvExport(nFormat:integer;const FName:PAnsiChar;hSp:HSPEC):integer;
var
  cnv:TBaseConv;
begin
  Result:=CNV_FAIL;
  cnv:=CnvFact.CreateConv(nFormat);
  try
    Result:=cnv.Exp(hSp,FName);
  finally
    cnv.Free;
  end;
end;


{ TConvFactory }

constructor TConvFactory.Create;
begin
  fList:=TList.Create;
end;

destructor TConvFactory.Destroy;
begin
  fList.Free;
end;

procedure TConvFactory.Register(const ref: TbaseConvClass);
begin
  if fList.IndexOf(ref)<0 then
  fList.Add(ref);
end;

procedure TConvFactory.Register(const ref: array of TbaseConvClass);
var
  i:integer;
begin
  for i:=low(ref) to high(ref) do
  Register(ref[i]);
end;

function TConvFactory.FormatCount: integer;
begin
  Result:=fList.Count;
end;

function TConvFactory.FormatName(ndx: integer): utf8string;
begin
  if (ndx>0) and (ndx<=fList.Count) then
    Result:=TBaseConvClass(fList[ndx-1]).FormatName
  else
    Result:='';
end;

function TConvFactory.FormatFlags(ndx: integer): integer;
begin
  if (ndx>0) and (ndx<=fList.Count) then
    Result:=TBaseConvClass(fList[ndx-1]).FormatFlags
  else
    Result:=0;
end;

function TConvFactory.CreateConv(nFormat: integer): TBaseConv;
begin
  if (nFormat>0) and (nFormat<=fList.Count) then
  Result:=TbaseConvClass(fList[nFormat-1]).Create
  else
  Result:=nil;
end;

initialization
finalization
  if Assigned(ffact) then
  ffact.Free;
end.

