unit devtypes;
interface
uses types;
type BOOL=longbool;

const
	// device status
	dsNormal=0;
	dsIdle=dsNormal;

	dsPreparing=1;
	dsPrerun=dsPreparing;

	dsStarted=2;
	dsStarting=dsStarted;
	dsRun=dsStarted;

	dsPostrun=3;
	dsCooling=4;

	dsShowMisc=128;

	// device workfile locking/update
	dlBeginUpdate = 1;
	dlEndUpdate 	= 0;
	dlCancelUpdate	= -1;

	// in-data status bits (MSB byte in 4-byte data) for block callback
	bdsStarting = $20;
	ldsStarting = $20000000;
	bdsRange    = $40;
	ldsRange    = $40000000;


	// New GC/LC zone indices
	zNone   = -1;
  zSystem = zNone; // LC system parameters
	zOven   = 0;     // GC or LC column temperature
	zInj    = $10000;
	zInjA   = zInj;
	zInjB   = zInj + 1;
	zInjC   = zInj + 2;
	zInjD   = zInj + 3;
	zDet    = $20000; // GC and LC detectors
	zDetA   = zDet;
	zDetB   = zDet + 1;
	zDetC   = zDet + 2;
	zDetD   = zDet + 3;
	zAux    = $30000;
	zAux1   = zAux;
	zAux2   = zAux + 1;
	zAux3   = zAux + 2;
  // "New" LC zone indices
  zPump   = zInj;
  zPumpA  = zInjA;
  zPumpB  = zInjB;
  zPumpC  = zInjC;
  zPumpD  = zInjD;
  zSample = zAux;  // autosampler

	zValueType  = $7FFF0000;
	zValueIndex = $FFFF;
	zTypeShift  = 16;

	ztOven   = 0;
	ztInj    = 1;
	ztDet    = 2;
	ztAux    = 3;
	ztSystem = -1;

	ztColumn = ztOven;
  ztPump   = ztInj;
  ztSample = ztAux;


(*      | DetA      InjA  InjB  DetB
   -----+-------------------------------
   Gaz1 | MakeUp    Car   Car   MakeUp
   Gaz2 | Hydrogen  Spl   Spl   Hydrogen
   Gaz3 | Air       Sept  Sept  Air
*)

  // New GC zoneObject indices
  zoNone     = -1;

	zoTemp     = 0;

	zoSelf		 = 0; // self zone index for entire zone 

  zoFlow     = $10000;
  zoFlowA    = zoFlow;
  zoFlowB    = zoFlow + 1;
  zoFlowC    = zoFlow + 2;

  zoPres     = $20000;
  zoPresA    = zoPres;
  zoPresB    = zoPres + 1;
  zoPresC    = zoPres + 2;

  zoVel      = $30000;
  zoVelA     = zoVel;
  zoVelB     = zoVel + 1;
  zoVelC     = zoVel + 2;

  zoCol      = $40000;
  zoColA     = zoCol;
  zoColB     = zoCol + 1;
  zoColC     = zoCol + 2;

  zoVol      = zoVel; //for LC

	zotTemp    = 0;
	zotSelf 	 = 0;
		
  zotFlow    = 1;
	zotPres    = 2;
  zotVel     = 3;
  zotCol     = 4;

  zotVol     = zotVel; //for LC
  zotScan    = zotFlow;

  // New GC objectParameter indices
  opNone    = -1;
  opMin     = 0;
  opMax     = 1;
  opSet     = 2;
	opCur     = 3;
  opDrift   = 4;
  opGas     = 5;
  opProg    = 6;
  opName    = 7;
  opLight   = 8;
  opEnabled = 9;
  opMode    = 10;
  opColLen  = 11;
  opColIDm  = 12;
  opType    = 13;
	opTime    = 14; // time from the start in milliseconds
	// For LC
	opPumpSet    = opSet; // Pumping flow
	opLoadSet    = 15;    // Loading eluent flow
	opEluent     = opGas;
	opReset      = opNone;
	opPumpType   = opType;  // type of a pump - 0 - plunger 1 - syringe (ptPlunger, ptSyringe)
	opPumpState  = opMode;  // Pump State: 0=Stopped, 1=Pumping, 2=Loading (psStopped etc)
												  // for system it's -1=disable, 0-run, 1-pause !!!!
                          // for column flow it's 0-run and 1-stop after program
	opDetOn      = opLight;
	opTurret     = 16;
	opChannel    = opTurret;
	opNeedle     = 17;
	opSigBkgrnd  = 18;
	opSigFiltre  = 19;
	opCalibrate  = 20;
	opScanMode   = 21;
	opPMTVoltage = 22;

	opPower			 = 23;// the power scale of units used for setting this device
	// i.e. returning 3 - means 10^3 = 1000
	// if the sensor returns InvalidZoneValue or 0 - then the multiplier is 10^0 = 1

	opStep       = opMode; //      :
												 // 0 -       
												 // 1 -     

	// New: gas types

	gtNone     = -1;
	gtUnknown  = 0;
	gtNitrogen = 1;
	gtArgon    = 2;
	gtHelium   = 3;
	gtHydrogen = 4;
	gtAir      = 5;
	gtOxygen   = 6;
	gtArgonMethane = 7;

	// Light state

	ltNone = -1;
	ltUnknown = 0;
	ltOff = 1;
	ltOn = 2;

  // any toggle switch state
  tsNo  = -1; // disable
  tsOff = 0;
  tsOn  = 1;

  // Pump type

	lcpPlunger = 0;
	lcpSyringe = 1;

  // Pump state
  psNone    = -1;
  psStopped = 0;
  psPumping = 1;
  psLoading = 2;


	// notify indexes
	// new and old also
	rnAll		=-1;
	rnStart	=-2;
	rnStop	=-3;
	rnReady	=-4;
	rnSeqFinish = -5;
	rnReconfig = -100;



	// Invalid values
	InvalidZoneValue = -1000;


type
	IHandle=THandle;
	// Device Info Structure
	TDeviceInfo=record
		DevType  :integer;
		FullScale:integer;
		DevMin   :single;
		DevMax   :single;
	end;
	//Additional types
	TIntArray=array[word]of Integer;
	PIntArray=^TIntArray;
	TDWordArray=array[word]of dword;
	PDWordArray=^TDWordArray;
	TQWordArray=array[word]of Int64;
	PQWordArray=^TQWordArray;
	TFloatArray=array[word]of Single;
	PFloatArray=^TFloatArray;
	TDoubleArray=array[word]of Double;
	PDoubleArray=^TDoubleArray;
	//Program callback types
	TPointCallback=Procedure(Self:pointer;data:integer;stat:integer);stdcall;
	TBlockCallback=Procedure(Self:pointer;PBuf:PIntArray;cbLen:integer);stdcall;
	TInt32Callback=procedure(Self:pointer;PData:PDwordArray;cbLen:integer;stat:integer);stdcall;
	TInt64Callback=procedure(Self:pointer;PData:PQwordArray;cbLen:integer;stat:integer);stdcall;
	TFloatCallback=procedure(Self:pointer;PData:PFloatArray;cbLen:integer;stat:integer);stdcall;
	TDoubleCallback=procedure(Self:pointer;PData:PDoubleArray;cbLen:integer;stat:integer);stdcall;

	TPageNotify=procedure(Client:pointer;npage:integer;hDevice:integer);stdcall;
	//new name
	TNotify=procedure(Client:pointer;AZone:integer;HInstrument:IHandle);stdcall;

	TDebugWriteStr = procedure(Client:pointer;PStr:PChar); stdcall;

	PObjectFunction=^TObjectFunction;
	TObjectFunction=packed record
		 AFunction,AClient:pointer;
	end;

	//Client Info Structure
	TPointClient=record
		Action:TPointCallback;
		Client:pointer;
		Profile:Integer;
		BaseAddr:Integer;
	end;
	TClient=record
		Action:TInt32Callback;
		Client:pointer;
	end;
	TClientEx=record
		Action:TInt32Callback;
		Client:pointer;
		Frequency:single;
	end;

	//Internal Device usage
	TCallbackList=array[word]of TPointClient;
	TPointCallbackList=TCallbackList;
	//TBlockCallbackList=array[word]of TBlockClient;
	PCallbackList=^TCallbackList;
	PPointCallbackList=^TPointCallbackList;
	//PBlockCallbackList=^TBlockCallbackList;
	TClients=array[word]of TClientEx;
	PClients=^TClients;

	PRamp=^TRamp;
	TRamp=record
		ValRate,FinalVal,IsoTime:single;
	end;

	PFlowRec=^TFlowRec;
	TFlowRec=packed record {deprecated}
		index:integer;
		Time,Flow,Flow1,Flow2,Flow3,Flow4:double;
	end;

  PPumpRamp=^TPumpRamp;
  TPumpRamp=packed record
    Time,Flow:double;
    case integer of
      0: (Flow1,Flow2,Flow3,Flow4:double);
      1: (Flows:array[0..3] of double);
  end;

  PRampEx=^TRampEx;
  TRampEx=packed record
    case integer of
    0: (gc:TRamp);
    1: (lc:TPumpRamp);
    2: (i:integer);
    3: (f:double);
  end;

	//new instrumental functions
	TdrvOpenEx=function(ADebugFunction:TDebugWriteStr;AClient:Pointer):BOOL;stdcall;
	TdrvOpenInstrument=function(AProfile:integer;AnAction:TPageNotify;AClient:pointer):integer;stdcall;
	TdrvCloseInstrument=function(HInstrument:integer):BOOL;stdcall;
	TdrvOpenPointChannelEx=function(HInstrument:IHandle;AnAction:TPointCallback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvOpenBlockChannelEx=function(HInstrument:IHandle;AnAction:TBlockCallback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvOpenInt32ChannelEx=function(HInstrument:IHandle;AnAction:TInt32Callback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvOpenInt64ChannelEx=function(HInstrument:IHandle;AnAction:TInt64Callback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvOpenFloatChannelEx=function(HInstrument:IHandle;AnAction:TFloatCallback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvOpenDoubleChannelEx=function(HInstrument:IHandle;AnAction:TDoubleCallback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	TdrvCloseChannelEx=function(HInstrument:IHandle;AChannel:integer):BOOL;stdcall;
	TdrvGetChannelInfo=function(HInstrument:IHandle;AChannel:integer;var DevInfo:TDeviceInfo):BOOL;stdcall;
	TdrvGetChannelFrequency=function(HInstrument:IHandle;AChannel:integer):single;stdcall;
	TdrvSetChannelFrequency=function(HInstrument:IHandle;AChannel:integer;freq:single):single;stdcall;
	TdrvChannelCalibrate=function(HInstrument:IHandle;AChannel:integer;Adata:double):double;stdcall;
	TdrvSetState=function(HInstrument:IHandle;AState:integer):BOOL;stdcall;
	TdrvGetState=function(HInstrument:IHandle):integer;stdcall;

	TdrvGetZoneInfo=function (HInstrument:IHandle;AZone,AZoneObject:integer;lpBuf:PChar;var cb:Integer):boolean;stdcall;
	TdrvUpdate=function(HInstrument:IHandle;AFlag:integer):boolean;stdcall;

	TtcbStartStop=TdrvSetState;

	// new temp api
	TtcbSetZoneValueEx=function (HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer;AValue:extended):BOOL;stdcall;
	TtcbGetZoneValueEx=function (HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):extended;stdcall;
	TtcbCheckReadZoneValue=function(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
	TtcbCheckWriteZoneValue=function(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
	TtcbSetProgRamp=function (HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;
	TtcbGetProgRamp=function (HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;

  // new unified tcb/fcb functions
	TdrvSetProgRampEx=function (HInstrument:IHandle;AZone,AZoneObject,ARampIndex,ARampSize:integer;PRec:PRampEx):BOOL;stdcall;
	TdrvGetProgRampEx=function (HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):BOOL;stdcall;

	// Flow api
	TfcbInitialize=TdrvOpenInstrument;
	TfcbDone=TdrvCloseInstrument;
	TfcbSetRec=function (hnd,ndx:integer;Prec:PFlowRec):BOOL;stdcall;

	// new flow api
	TfcbSetRecEx=function (HInstrument:IHandle;AZone,AZoneObject:integer;PRec:PFlowRec):BOOL;stdcall;

///////////////////////////////////
// the very new Autosampler API
///////////////////////////////////

type
	TsmpGetParam=function (HInstrument:IHandle;ATower,AParamIndex:integer):integer;stdcall;
	TsmpSetParam=function (HInstrument:IHandle;ATower,AParamIndex,AParam:integer):bool;stdcall;

	TsmpSetSample=function (HInstrument:IHandle;ASamp,ATower,AVial,ACount,AVolume:integer):boolean;stdcall;
	TsmpGetSample=function (HInstrument:IHandle;ASamp,AParamIndex:integer):integer;stdcall;

	TevtGetParam=function (HInstrument:IHandle;AEvent,AParamIndex:integer):integer;stdcall;
	TevtSetParam=function (HInstrument:IHandle;AEvent,AParamIndex,AParam:integer):bool;stdcall;

const
	// indices for getting parameters of
	// choosen sample
	// e.g. smpGetParam(hInst,1,1,smpiPos) - get the tray position of 1-st sample in sequence
	// samples are 1-bases
	// towers are 1-based
	twiAny = -1;
	ssAny  = twiAny;

	twiValves = -2;

	// used for getting information on sampling system in total
 (* smpnInfo=-1;*)
	// indices for getting total information on sampling system
	// smpGetParam(hInst,twiAny,smpnInfo,smpiStatus)   - get the number of installed towers
	// smpGetParam(hInst,2,smpnInfo,smpiStatus) - get the status of second tower
	smpiStatus   = -1;  //the number of installed towers
	smpiTowerMax = -2; // the number of positions for tower
	smpiTotal		 = -3; // total number of samples

	smpiInjectionSource = 1;

		smpManual = 0;
		smpALS		= 1;
		smpValve	= 2;

	// Sampling system parameters
  smpiSolventWash = -4; // number or time of washing
  smpiSamplePump	= -5; // number or time of sample pumping
  smpiInjSpeed		= -6; // speed or time of sample injection
  smpiLoadSpeed		= -7;	// speed or time of sample loading
  smpiDwellTime		= -8; // dwelling time in injector

  // HP6890-added parameters
  smpiSolventAPreWash    = smpiSolventWash;
  smpiSolventBPreWash    = -9; //number or time of washing...
  smpiSolventAPostWash   = -10; // ...in two solvents...
  smpiSolventBPostWash   = -11; // ...before and after injection

  smpiSamplePreWash      = -12; // washing in sample before injection??
  //smpiSamplePump       = -5; // number or time of sample pumping
  //smpiInjSpeed         = -6; // speed or time of sample injection (0 or 1 for HP)
  smpiViscosityDelay     = smpiLoadSpeed;
  smpiPreInjectionDwell  = smpiDwellTime; // dwelling times in injector...
  smpiPostInjectionDwell = -13; // before and after injection, in 0.01 mins (0-100)
  smpiSampleSkimDepth    = -14; // skim??

  // indices for getting sample parameters
  smpiTower		 	= 0; // max number of samples in sequence
  smpiPos		 		= 1; // position of chosen sample in tray
  smpiVolume 		= 2; // injection volume of chosen sample
  smpiCount			= 3; // number of injection

 (* // genreric get/set sample info

  smpiPos		 		= 1; // position of chosen sample in tray
  smpiVolume 		= 2; // injection volume of chosen sample
  smpiCount			= 3; // number of injection
  smpiMax		 		= 4; // max number of samples in sequence*)

	// generic get/set valve info
  vpiValve   = 1000;  // number of referenced valve
  vpiType    = 1001;  // type of referenced valve
  vpiInlet   = 1002;  // inlet to which referenced valve is connected
  vpiState   = 1003;  // state of referenced valve
  vpiTime    = 1004;  // time of valve event
  vpiVolume  = 1005;  // loop volume
  vpiLoadTime   = 1006;  // time for blowing sample loop with sample gas
  vpiInjectTime = 1007;  // time for connecting sample loop to inlet
	vpiCount	 = -1;


	function MapIndices(AZone,AZoneObject{,AParamIndex}:integer; var AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer):boolean;
implementation

function MapIndices(AZone,AZoneObject{,AParamIndex}:integer; var AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer):boolean;
begin
	if AZone<0
	then begin
		AZoneType:=-1;
		AZoneIndex:=-1;
	end
	else begin
		AZoneType:=AZone shr zTypeShift;
		AZoneIndex:=AZone and zValueIndex;
	end;
	if AZoneObject<0
	then begin
		AObjectType:=-1;
		AObjectIndex:=-1;
	end
	else begin
		AObjectType:=AZoneObject shr zTypeShift;
		AObjectIndex:=AZoneObject and zValueIndex;
	end;
	Result:=true;
end;

end.
