#ifndef _SPEC_API_

#define _SPEC_API_

#ifdef __cplusplus
extern "C" {
#endif

// spectrum generic api
typedef unsigned int HSpec;
typedef unsigned int HPeak;

const unsigned int HINVALID=NULL;
const int ACTIVE_SPEC=-1;
const int GET_LOCAL = true;
const int GET_GLOBAL = false;

const int PAR_INT=0;
const int PAR_FLOAT=1;
const int PAR_STR=2;


// peak information Mask sequentially

const int piName	= 0x00000001;
const int piX            	= 0x00000002;
const int piTimeIndex     = 0x00000004;
const int piPeakIndex     = 0x00000008;
const int piMass          = 0x00000010;
const int piBoilTemp      = 0x00000020;
const int piDensity       = 0x00000040;
const int piXLeft         = 0x00000080;
const int piXRight        = 0x00000100;
const int piArea          = 0x00000200;
const int piAmplitude     = 0x00000400;
const int piFactor        = 0x00000800;
const int piConc          = 0x00001000;
const int piMassConc      = 0x00002000;
const int piMolarConc     = 0x00004000;
const int piTitre         = 0x00008000;
const int piMolarity      = 0x00010000;
const int piState         = 0x00020000;

//----------- intensive params -----------------
const int piPlates        = 0x00040000;
const int piEffPlates     = 0x00080000;
const int piHeightETP     = 0x00100000;
const int piTailing5      = 0x00200000;
const int piHalfWidth     = 0x00400000;
const int piExtraction    = 0x00800000;
const int piResToNext     = 0x01000000;
const int piTailing10     = 0x02000000;
const int piAsymmetry			= 0x04000000;
const int piAlpha					= 0x08000000;
const int piBeta					= 0x10000000;

const int piTotal         = 0x0FFFFFFF;


// peak states
const int pmReper					= 1;
const int pmCalibPoint		= 2;
const int pmGaussFit			= 4;
const int pmEmgFit				= 8;
const int pmNoReport			= 16;
const int pmFictive				= 32;


#ifdef __GNUC__

typedef char* PCHAR;
    #define SPEC_API __attribute__((stdcall))
    
#else
    #define SPEC_API __declspec(dllimport) __stdcall
#endif

// spectrum management
// return the number of opened spectrums
int SPEC_API GetNspec();

// returns the spectrum by its number
HSpec SPEC_API GetAspec(int num);

// find a spectrum by its name
HSpec SPEC_API FindASpec(char* Aname);

// open a spectrum from disk
HSpec SPEC_API OpenSpec(char* FileName);

// close a selected spectrum
bool SPEC_API CloseSpec(HSpec Sp);

// properties management
// get number of local or global properties
int SPEC_API GetNumProp(HSpec Sp, bool isLocal);

// get property by its index
bool SPEC_API GetPropByNdx(HSpec Sp,bool isLocal,int ndx,int cbbuf,void* buf);

// set property by its index
bool SPEC_API SetPropByNdx(HSpec Sp,bool isLocal,int ndx,PCHAR Val);

// get property by its name
bool SPEC_API GetPropByName(HSpec Sp,bool isLocal,PCHAR Aname, int cbbuf, void* buf);


bool SPEC_API GetPropByNameEx(HSpec Sp,bool isLocal, PCHAR Aname, int cbbuf,int* buftype,void* buf);

// if nlay=-1 return global property
bool SPEC_API GetPropValue(HSpec Sp, int nlay, PCHAR Aname, VARIANT* val);

bool SPEC_API SetPropValue(HSpec Sp, int nlay, PCHAR Aname, VARIANT val);

// set property by its name
bool SPEC_API SetPropByName(HSpec Sp,bool isLocal,PCHAR Aname,PCHAR Val);
//function SetPropByNameEx(Sp:HSpec;isLocal:bool;Aname:PChar;cbbuf,buftype:integer;const buf):bool;stdcall;

// peaks and lays management

// get number of layers
int SPEC_API GetNLays(HSpec Sp);

// get number of peak in layer
int SPEC_API GetNPeaks(HSpec Sp, int lay);

// get peak handle
HPeak SPEC_API GetPeak(HSpec Sp,int lay,int ndx);

// get parameter of peak
bool SPEC_API GetPeakParam(HPeak Apk, int parndx, int* buftype, void* buf);

// set peak parameter
bool SPEC_API SetPeakParam(HPeak Apk, int parndx, int buftype, const void* buf);

// Data Layer management
//
bool SPEC_API SetDataLayer(HSpec Asp, int aLay, int nPoints, int nPointSize, const void* buf);

bool SPEC_API GetDataLayer(HSpec Asp, int aLay, int* nPoints, int* nPointSize,void* buf);

bool SPEC_API SetData(HSpec Asp, int aLay, int ndx, double AData);

double SPEC_API GetData(HSpec Asp, int aLay, int ndx);

HPeak SPEC_API AddPeak(HSpec Asp, int aLay,int l,int c, int r);


bool SPEC_API StoreToStreamHandle(const int AHandle,const HSpec ASpec);
bool SPEC_API LoadFromStreamHandle(const int AHandle,const HSpec ASpec);

// picture
bool SPEC_API GetPicture(const HSpec ASpec,int lay,int mmWidth,int mmHeight);






#if defined(__cplusplus)
}
#endif

#endif
