#ifndef _UC_API_

#define _UC_API_

#ifdef __cplusplus
extern "C" {
#endif

const int SPEC_API_VERSION=1;

// spectrum generic api
typedef void* HSpec;
typedef void* HPeak;
typedef void* HSample;
typedef void* HObj;

#define HINVALID NULL;
const int ACTIVE_SPEC=-1;
const int GET_LOCAL = true;
const int GET_GLOBAL = false;

const int PAR_INT=0;
const int PAR_FLOAT=1;
const int PAR_STR=2;


// peak information Mask sequentially

const int piName	= 0x00000001;
const int piX            	= 0x00000002;
const int piTimeIndex     = 0x00000004;
const int piPeakIndex     = 0x00000008;
const int piMass          = 0x00000010;
const int piBoilTemp      = 0x00000020;
const int piDensity       = 0x00000040;
const int piXLeft         = 0x00000080;
const int piXRight        = 0x00000100;
const int piArea          = 0x00000200;
const int piAmplitude     = 0x00000400;
const int piFactor        = 0x00000800;
const int piConc          = 0x00001000;
const int piMassConc      = 0x00002000;
const int piMolarConc     = 0x00004000;
const int piTitre         = 0x00008000;
const int piMolarity      = 0x00010000;
const int piState         = 0x00020000;

//----------- intensive params -----------------
const int piPlates        = 0x00040000;
const int piEffPlates     = 0x00080000;
const int piHeightETP     = 0x00100000;
const int piTailing5      = 0x00200000;
const int piHalfWidth     = 0x00400000;
const int piExtraction    = 0x00800000;
const int piResToNext     = 0x01000000;
const int piTailing10     = 0x02000000;
const int piAsymmetry			= 0x04000000;
const int piAlpha					= 0x08000000;
const int piBeta					= 0x10000000;

const int piTotal         = 0x0FFFFFFF;


// peak states
const int pmReper					= 1;
const int pmCalibPoint		= 2;
const int pmGaussFit			= 4;
const int pmEmgFit				= 8;
const int pmNoReport			= 16;
const int pmFictive				= 32;


#ifdef __GNUC__

typedef char* PCHAR;
  #ifndef __x86_64 // amd64 architecture do not use stdcall
    #define SPEC_API __attribute__((stdcall))
  #else
    #define SPEC_API
  #endif
#else
    #define SPEC_API  __stdcall
#endif

// spectrum management
// return the number of opened spectrums
typedef int (SPEC_API *TGetNspec)(void);

// returns the spectrum by its number
typedef HSpec (SPEC_API *TGetAspec)(int num);

// crete new spectrum
typedef HSpec (SPEC_API *TCreateSpec)();

// find a spectrum by its name
typedef HSpec (SPEC_API *TFindASpec)(char* Aname);

// open a spectrum from disk
typedef HSpec (SPEC_API *TOpenSpec)(char* FileName);

// close a selected spectrum
typedef bool (SPEC_API *TCloseSpec)(const HSpec ASp);

// saving of method
typedef int (SPEC_API *TSaveSpec)(const HSpec ASp);


// properties management
// get number of local or global properties
typedef int (SPEC_API *TGetNumProp)(HSpec Sp, bool isLocal);

// get property by its index
typedef bool (SPEC_API *TGetPropByNdx)(HSpec Sp,bool isLocal,int ndx,int cbbuf,void* buf);

// set property by its index
typedef bool (SPEC_API *TSetPropByNdx)(HSpec Sp,bool isLocal,int ndx,PCHAR Val);

// get property by its name
typedef bool (SPEC_API *TGetPropByName)(HSpec Sp,bool isLocal,PCHAR Aname, int cbbuf, void* buf);


typedef bool (SPEC_API *TGetPropByNameEx)(HSpec Sp,bool isLocal, PCHAR Aname, int cbbuf,int* buftype,void* buf);

// if nlay=-1 return global property
typedef bool (SPEC_API *TGetPropValue)(HSpec Sp, int nlay, PCHAR Aname, VARIANT* val);

typedef long (SPEC_API *TSetPropValueEx)(HSpec Sp, int nlay, PCHAR Aname, const VARIANT* val);

typedef long (SPEC_API *TSetPropValue)(HSpec Sp, int nlay, PCHAR Aname, VARIANT val);


// set property by its name
typedef long (SPEC_API *TSetPropByName)(HSpec Sp,bool isLocal,PCHAR Aname,PCHAR Val);
//function SetPropByNameEx(Sp:HSpec;isLocal:bool;Aname:PChar;cbbuf,buftype:integer;const buf):bool;stdcall;

// peaks and lays management

// get number of layers
typedef int (SPEC_API *TGetNLays)(HSpec Sp);

// get number of peak in layer
typedef int (SPEC_API *TGetNPeaks)(HSpec Sp, int lay);

// get peak handle
typedef HPeak (SPEC_API *TGetPeak)(HSpec Sp,int lay,int ndx);

// get parameter of peak
typedef bool (SPEC_API *TGetPeakParam)(HPeak Apk, int parndx, int* buftype, void* buf);
typedef bool (SPEC_API *TGetPeakParamEx)(HPeak Apk, int parndx, VARIANT* buf);

// set peak parameter
typedef bool (SPEC_API *TSetPeakParam)(HPeak Apk, int parndx, int buftype, const void* buf);
typedef bool (SPEC_API *TSetPeakParamEx)(HPeak Apk, int parndx, const VARIANT buf);



// Data Layer management
//
typedef bool (SPEC_API *TSetDataLayer)(HSpec Asp, int aLay, int nPoints, int nPointSize, const void* buf);

typedef bool (SPEC_API *TGetDataLayer)(HSpec Asp, int aLay, int* nPoints, int* nPointSize,void* buf);

typedef bool (SPEC_API *TSetData)(HSpec Asp, int aLay, int ndx, double AData);

typedef double (SPEC_API *TGetData)(HSpec Asp, int aLay, int ndx);

typedef HPeak (SPEC_API *TAddPeak)(HSpec Asp, int aLay,int l,int c, int r);

// Noise
typedef double (SPEC_API *TGetFNoise)(HSpec Asp,int aLay,double hw,double fb,double fe);
typedef double (SPEC_API *TGetFNoiseBand)(HSpec Asp,int aLay,double hw,double fb,double fe);
typedef double (SPEC_API *TGetFNoiseEx)(const HSpec Asp, int aLay, double hw,double fstep, double fb, double fe, int nType);

typedef bool (SPEC_API *TStoreToStreamHandle)(const int AHandle,const HSpec ASpec);
typedef bool (SPEC_API *TLoadFromStreamHandle)(const int AHandle,const HSpec ASpec);

// picture
typedef bool (SPEC_API *TGetPicture)(const HSpec ASpec,int lay,int mmWidth,int mmHeight);


// samples management
typedef int (SPEC_API *TGetNSmpLists)(const HSpec ASpec);
typedef int (SPEC_API *TGetNSamples)(const HSpec ASpec,int nlist);
typedef HSample (SPEC_API *TGetSample)(const HSpec ASpec, int nlist,int nsample);

// object properties management
typedef int (SPEC_API *TGetObjProp)(const HObj AObj, const PCHAR APropName,VARIANT* val);
typedef int (SPEC_API *TSetObjProp)(const HObj AObj, const PCHAR APropName,const VARIANT val);

// Calibration table
typedef int (SPEC_API *TGetCalibData)(const HSpec ASpec,const wchar_t* ACalibName,const int npar,VARIANT* res);
typedef int (SPEC_API *TSetCalibData)(const HSpec ASpec,const wchar_t* ACalibName,const int npar,const VARIANT val);


//  every addition should take place at the end of table
// every interface modification must increment the version number

typedef
  struct TUniChromEntryTable
	{
    int dwSize;
    int dwVersion;
    // specapi interface
    TGetNspec GetNspec;
    TGetAspec GetAspec;
    TCreateSpec CreateSpec;
    // find a spectrum by its name
    TFindASpec FindASpec;
    // open a spectrum from disk
    TOpenSpec OpenSpec;
    // close a selected spectrum
		TCloseSpec CloseSpec;
    // get number of local or global properties
    TGetNumProp GetNumProp;
    // get property by its index
    TGetPropByNdx GetPropByNdx;
    // set property by its index
    TSetPropByNdx SetPropByNdx;
    // get property by its name
    TGetPropByName GetPropByName;
    TGetPropByNameEx GetPropByNameEx;
    TGetPropValue GetPropValue;
    TSetPropValue SetPropValue;
    // set property by its name
    TSetPropByName SetPropByName;
    // get number of layers
    TGetNLays GetNLays;
    // get number of peak in layer
    // "lay" is an index of lay but not it's a number
    TGetNPeaks GetNPeaks;
    // get peak handle
    TGetPeak GetPeak;
    // get parameter of peak
    TGetPeakParam GetPeakParam;
    TGetPeakParamEx GetPeakParamEx;
    // set peak parameter
    TSetPeakParam SetPeakParam;
    TSetPeakParamEx SetPeakParamEx;
    // Data Layer management
    TSetDataLayer SetDataLayer;
    TGetDataLayer GetDataLayer;
    TSetData SetData;
    TGetData GetData;
    // peak management
    TAddPeak AddPeak;
    // Noise
    TGetFNoise GetFNoise;
    TGetFNoiseBand GetFNoiseBand;
    // save or load by handle
    TStoreToStreamHandle StoreToStreamHandle;
    TLoadFromStreamHandle LoadFromStreamHandle;
    // picture
    TGetPicture GetPicture;

		// new noise calculation routine
    TGetFNoiseEx GetFNoiseEx;

    // samples management
    TGetNSmpLists GetNSmpLists;
    TGetNSamples GetNSamples;
    TGetSample GetSample;

    // object properties management
    TGetObjProp GetObjProp;
    TSetObjProp SetObjProp;

    // Calibration Table
    TGetCalibData GetCalibData;
    TSetCalibData SetCalibData;

    // new Saving function
    TSaveSpec SaveSpec;

    // x-data function
    TGetData GetXData;
    TSetData SetXData;

    TSetPropValueEx SetPropValueEx;
}
	*PUniChromEntryTable;

typedef struct TUnicodeRec
{
    size_t ref;
    size_t len;
    WCHAR first;
} *PUnicodeRec;

#if defined(__cplusplus)
}
#endif

#endif
