unit ucintf;

interface

// spectrum generic api
type
  HObj=type pointer;
	HSpec=type HObj;
	HPeak=type HObj;
  HSample=type HObj;
  
  {$ifndef fpc}
  PtrInt={$ifdef cpu64}int64{$else}longint{$endif};
  PtrUInt={$ifdef cpu64}int64{$else}longword{$endif};
  {$endif}


const
      HINVALID=nil;
			ACTIVE_SPEC=-1;
			GET_LOCAL = true;
			GET_GLOBAL = false;

			PAR_INT=0;
			PAR_FLOAT=1;
			PAR_STR=2;

  {peak information Mask sequentially}

	 piName          = $00000001;
	 piX             = $00000002;
	 piTimeIndex     = $00000004;
	 piPeakIndex     = $00000008;
	 piMass          = $00000010;
	 piBoilTemp      = $00000020;
	 piDensity       = $00000040;
	 piXLeft         = $00000080;
	 piXRight        = $00000100;
	 piArea          = $00000200;
	 piAmplitude     = $00000400;
	 piFactor        = $00000800;
	 piConc          = $00001000;
	 piMassConc      = $00002000;
	 piMolarConc     = $00004000;
	 piTitre         = $00008000;
	 piMolarity      = $00010000;
	 piState         = $00020000;
	 //----------- intensive params -----------------
	 piPlates        = $00040000;
	 piEffPlates     = $00080000;
	 piHeightETP     = $00100000;
	 piTailing5      = $00200000;
	 piHalfWidth     = $00400000;
	 piExtraction    = $00800000;
	 piResToNext     = $01000000;
	 piTailing10     = $02000000;
	 piAsymmetry     = $04000000;
	 piAlpha         = $08000000;
	 piBeta          = $10000000;

	 piTotal         = $1FFFFFFF;


   SPEC_API_VERSION=5;

type
  TPeakMode=(pmReper,pmCalibPoint,pmGaussFit,pmEmgFit,pmNoReport,pmFictive);
  TPeakState=set of TPeakMode;


// specapi interface prototypes
  TGetNspec = function :integer;stdcall;
  TGetAspec = function (num:integer):HSpec;stdcall;
  TCreateSpec = function : hSpec; stdcall;
  // find a spectrum by its name
  TFindASpec = function (Aname:Pchar):HSpec;stdcall;
  // open a spectrum from disk
  TOpenSpec = function (FileName:PChar):HSpec;stdcall;
  // close a selected spectrum
  TCloseSpec = function (Sp:HSpec):longbool;stdcall;

  TSaveSpec = function (Sp:HSpec):integer;stdcall;

  // get number of local or global properties
  TGetNumProp = function (Sp:HSpec;isLocal:longbool):integer;stdcall;
  // get property by its index
  TGetPropByNdx = function (Sp:Hspec;isLocal:longbool;Ndx,cbbuf:integer;var buf):longbool;stdcall;
  // set property by its index
  TSetPropByNdx = function (Sp:Hspec;isLocal:longbool;Ndx:integer;Val:PChar):longbool;stdcall;
  // get property by its name
  TGetPropByName = function (Sp:HSpec;isLocal:longbool;Aname:PChar;cbbuf:integer;var buf):longbool;stdcall;
  TGetPropByNameEx = function (Sp:HSpec;isLocal:longbool;Aname:PChar;cbbuf:integer;var buftype:integer;var buf):longbool;
  // GetPropValue   , 
  //    ,     , ..
  // (ASpec <> HINVALID) and (ASpec is destroyed)
  //  Exception "Access violation"
  TGetPropValue = function (Sp:HSpec;nlay:integer;Aname:PChar;var val:Variant):longbool;stdcall;
  TSetPropValue = function (Sp:HSpec;nlay:integer;Aname:PChar;val:Variant):longbool;stdcall;
  TSetPropValueEx = function (Sp:HSpec;nlay:integer;Aname:PChar;const val:PVariant):longbool;stdcall;
  // set property by its name
  TSetPropByName = function (Sp:HSpec;isLocal:longbool;Aname:PChar;Val:PChar):longbool;stdcall;
  // get number of layers
  TGetNLays = function (Sp:HSpec):integer;stdcall;
  // get number of peak in layer
  // "lay" is an index of lay but not it's a number
  TGetNPeaks = function (Sp:HSpec;lay:integer):integer;stdcall;

  // get peak handle
  TGetPeak = function (Sp:HSpec;lay,ndx:integer):HPeak;stdcall;
  // get parameter of peak
  TGetPeakParam = function (Apk:Hpeak;parndx:integer;var buftype:integer;var buf):longbool;stdcall;
  TGetPeakParamEx = function (Apk:Hpeak;parndx:integer;var buf:Variant):longbool;stdcall;
  // set peak parameter
  TSetPeakParam = function (Apk:Hpeak;parndx,buftype:integer;const buf):longbool;stdcall;
  TSetPeakParamEx = function (Apk:Hpeak;parndx:integer;const buf:variant):longbool;stdcall;

  // Data Layer management
  //
  TSetDataLayer = function (Asp:HSpec;aLay,nPoints,nPointSize:integer;var buf):boolean;stdcall;
  TGetDataLayer = function (Asp:HSpec;aLay:integer;var nPoints,nPointSize:integer;var buf):boolean;stdcall;
  TSetData = function (Asp:HSpec;aLay,ndx:integer;AData:double):boolean;stdcall;
  TGetData = function (Asp:HSpec;aLay:integer;ndx:integer):double;stdcall;

  // peak management
  TAddPeak = function (Asp:Hspec;aLay:integer;l,c,r:integer):HPeak;stdcall;

  // Noise
  TGetFNoise = function (Asp:Hspec;aLay:integer;hw,fb,fe:double):double;stdcall;
  TGetFNoiseBand = function (Asp:Hspec;aLay:integer;hw,fb,fe:double):double;stdcall;
  TGetFNoiseEx = function (Asp:Hspec;aLay:integer;hw,fstep,fb,fe:double;nType:integer):double;stdcall;

  // save or load by handle
  TStoreToStreamHandle = function (const AHandle:Thandle;ASpec:HSpec):boolean;stdcall;
  TLoadFromStreamHandle = function (const AHandle:Thandle;ASpec:HSpec):boolean;stdcall;

  // picture
  TGetPicture = function (ASpec:HSpec;lay,mmWidth,mmHeight:integer):boolean;stdcall;

  // samples management
  TGetNSmpLists = function(ASpec:HSpec):integer;stdcall;
  TGetNSamples = function(ASpec:HSpec;nlist:integer):integer;stdcall;
  TGetSample = function(ASpec:HSpec;nlist,nsample:integer):HSample;stdcall;

  // object properties management
  TGetObjProp = function(AObj:HObj;APropName:PChar;var val:Variant):integer;stdcall;
  TSetObjProp =function(AObj:HObj;APropName:PChar;const val:Variant):integer;stdcall;

  // Calibration table
  TGetCalibData = function (ASpec:HSpec;const ACalibName:PWideChar;npar:integer; var res:Variant):integer;stdcall;
  TSetCalibData = function (ASpec:HSpec;const ACalibName:PWideChar;npar:integer;const val:Variant):integer;stdcall;

type

  //  every addition should take place at the end of table
  // every interface modification must increment the version number

  PUniChromEntryTable=^TUniChromEntryTable;

  TUniChromEntryTable=packed record
    dwSize:integer;
    dwVersion:integer;
    // specapi interface
    GetNspec:TGetNspec;
    GetAspec:TGetAspec;
    CreateSpec:TCreateSpec;
    // find a spectrum by its name
    FindASpec:TFindASpec;
    // open a spectrum from disk
    OpenSpec:TOpenSpec;
    // close a selected spectrum
    CloseSpec:TCloseSpec;
    // get number of local or global properties
    GetNumProp:TGetNumProp;
    // get property by its index
    GetPropByNdx:TGetPropByNdx;
    // set property by its index
    SetPropByNdx:TSetPropByNdx;
    // get property by its name
    GetPropByName:TGetPropByName;
    GetPropByNameEx:TGetPropByNameEx;
    // nlay is a number of layer 1..NLayers , nlay=-1 - get a property of a spectrum
    GetPropValue:TGetPropValue;
    // nlay is a number of layer 1..NLayers , nlay=-1 - get a property of a spectrum
    SetPropValue:TSetPropValue;
    // set property by its name
    SetPropByName:TSetPropByName;
    // get number of layers
    GetNLays:TGetNLays;
    // get number of peak in layer
    // "lay" is an index of lay but not it's a number
    GetNPeaks:TGetNPeaks;
    // get peak handle
    GetPeak:TGetPeak;
    // get parameter of peak
    GetPeakParam:TGetPeakParam;
    GetPeakParamEx:TGetPeakParamEx;
    // set peak parameter
    SetPeakParam:TSetPeakParam;
    SetPeakParamEx:TSetPeakParamEx;
    // Data Layer management
    SetDataLayer:TSetDataLayer;
    GetDataLayer:TGetDataLayer;
    SetData:TSetData;
    GetData:TGetData;
    // peak management
    AddPeak:TAddPeak;
    // Noise
    GetFNoise:TGetFNoise;
    GetFNoiseBand:TGetFNoiseBand;
    // save or load by handle
    StoreToStreamHandle:TStoreToStreamHandle;
    LoadFromStreamHandle:TLoadFromStreamHandle;
    // picture
    GetPicture:TGetPicture;
    // new noise calculation routine
    GetFNoiseEx:TGetFNoiseEx;

    // samples management
    GetNSmpLists:TGetNSmpLists;
    GetNSamples:TGetNSamples;
    GetSample:TGetSample;

    // object properties management
    GetObjProp:TGetObjProp;
    SetObjProp:TSetObjProp;

    // Calibration Table
    GetCalibData:TGetCalibData;
    SetCalibData:TSetCalibData;

    // new Saving function
    SaveSpec:TSaveSpec;

    // x-data function
    GetXData:TGetData;
    SetXData:TSetData;

    // additional set property value by pointer
    SetPropValueEx:TSetPropValueEx;
  end;


implementation

end.


