unit ad2001;

interface

uses devtypes, drv_common, SysUtils,Types,
     {$IFDEF MSWINDOWS}
     windows
     {$ENDIF}
     {$IFDEF LINUX}
     Libc
     {$ENDIF};

type

  TInstrument=class;
  TInstruments=array[byte]of TInstrument;
  PInstruments=^TInstruments;

  TThermoProgram=array of TRamp;
  TFlowProgram=array of TPumpRamp;

	TDriver=class
  private
    FModuleName:string;
    DebugFunction:TDebugWriteStr;
    DebugFunctionClient:Pointer;
		FInstruments:PInstruments;
		FInstrumentsSize:integer;
    function GetInstruments(index:integer):TInstrument;
  protected
    function GetUniqueHandle:IHandle;

    function GetDeviceName:string; virtual; abstract;
    procedure SetInstrumentsSize(AInstrumentsSize:integer);
  public
    constructor Create; virtual;
    destructor Destroy; override;
		function FindInstrumentByHandle(AHandle:IHandle):integer;
    procedure CloseAll;
    procedure DbgWrite(AMessage:String);
    //
    property ModuleName:string read FModuleName;
    property DeviceName:string read GetDeviceName;
    property InstrumentsSize:integer read FInstrumentsSize write SetInstrumentsSize;
    property Instruments[index:integer]:TInstrument read GetInstruments;
    //
    function OpenInstrument(AProfile:integer;AnAction:TNotify;AClient:pointer):IHandle;virtual;
		function CloseInstrument(HInstrument:IHandle):boolean;virtual;
    //
    function OpenChannel(HInstrument:IHandle;AnAction:TInt32Callback;AClient:pointer;AChannel:integer):boolean;virtual;
    function CloseChannel(HInstrument:IHandle;AChannel:integer):boolean;virtual;
    function GetChannelInfo(HInstrument:IHandle;AChannel:integer;var DevInfo:TDeviceInfo):boolean;virtual;
    function GetFrequency(HInstrument:IHandle;AChannel:integer):single;virtual;
    function SetFrequency(HInstrument:IHandle;AChannel:integer;freq:single):single;virtual;
    function Calibrate(HInstrument:integer;AChannel:integer;Adata:double):double;virtual;
		function GetState(HInstrument:IHandle):integer;virtual;
    function SetState(HInstrument:IHandle;AState:integer):boolean;virtual;
		// Unified GC/LC
    function GetZoneInfo(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;lpBuf:PChar;var cb:Integer):boolean;virtual;
    function SetZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer;AValue:extended):boolean;virtual;
		function GetZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):extended;virtual;
		function CheckReadZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean; virtual;
    function CheckWriteZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean; virtual;
  	function SetProgRampEx(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize:integer;PRec:PRampEx):boolean;virtual;
	  function GetProgRampEx(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):boolean;virtual;
    //Thermo
    function SetProgRamp(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;virtual; deprecated;
    function GetProgRamp(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;virtual; deprecated;
    //HPLC
		function SetFlowRec(HInstrument:IHandle;AZone,AIndex:integer;PRec:PFlowRec):boolean; deprecated;

		// samples
		function GetSampParam(HInstrument:IHandle;ATower,AParamIndex:integer):integer;
		function SetSampParam(HInstrument:IHandle;ATower,AParamIndex,AParam:integer):boolean;

		function SetSample(HInstrument:IHandle;ASamp,ATower,AVial,ACount,AVolume:integer):boolean;
		function GetSample(HInstrument:IHandle;ASamp,AParamIndex:integer):integer;

		// events
		function GetEventParam(HInstrument:IHandle;AEvent,AParamIndex:integer):integer;
		function SetEventParam(HInstrument:IHandle;AEvent,AParamIndex,AParam:integer):boolean;

		// update
		function Update(HInstrument:IHandle;AParam:integer):boolean;virtual;

	end;

	TInstrument=class
	private
		FDriver:TDriver;
		FProfile:integer;
		FHandle:IHandle;
		FAction:TNotify;
    FClient:Pointer;
    //
    FThread,FThreadID:THandle;
		FNeedExit: boolean;
    //
    FChannels:PClients;
    FChannelsSize:integer;
    FState,FNewState:integer;
    FSetState:boolean;
    FStateChangeTime:integer;
		//
    FDefaultFrequency:single;
  protected
    function OpenDevice:boolean; virtual;
		function OpenThread:boolean; virtual;
    procedure CloseDevice; virtual;
    procedure CloseThread; virtual;
    procedure SetChannelsSize(AChannelsSize:integer);
    function Work:integer;
    function WorkThread:integer; virtual; abstract;
    function IsActiveChannel(AChannel:integer):boolean;
    procedure IntSetState(AState:integer);
    function GetStateTime:integer;
    procedure CallPointCallback(AChannel:integer;data:integer;stat:integer);
    procedure CallBlockCallback(AChannel:integer;PBuf:PIntArray;cbLen:integer);
    procedure CallInt32Callback(AChannel:integer;PData:PDwordArray;cbLen:integer;stat:integer);
    procedure CallInt64Callback(AChannel:integer;PData:PQwordArray;cbLen:integer;stat:integer);
    procedure CallFloatCallback(AChannel:integer;PData:PFloatArray;cbLen:integer;stat:integer);
    procedure CallDoubleCallback(AChannel:integer;PData:PDoubleArray;cbLen:integer;stat:integer);
  public
		constructor Create(ADriver:TDriver; AProfile:integer; AnAction:TNotify; AClient: Pointer); virtual;
		destructor Destroy; override;
    procedure DbgWrite(AMessage:String);
    procedure CallNotify(AZone:integer);virtual;
    function ProfilePath:string;
    function OpenChannel(AnAction:TInt32Callback;AClient:pointer;AChannel:integer):boolean;virtual;
		function CloseChannel(AChannel:integer):boolean;virtual;
    function Calibrate(AChannel:integer;Adata:double):double;virtual;
    function GetFrequency(AChannel:integer):single;virtual;
    function SetFrequency(AChannel:integer;freq:single):single;virtual;
    function GetChannelInfo(AChannel:integer;var DevInfo:TDeviceInfo):boolean;virtual; abstract;
    function GetState:integer;virtual;
    function SetState(AState:integer):boolean;virtual;
		// Unified GC/LC
		function GetZoneInfo(AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;lpBuf:PChar;var cb:Integer):boolean;virtual; abstract;
    function SetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer;AValue:extended):boolean;virtual; abstract;
    function GetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):extended;virtual; abstract;
		function CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean; virtual; abstract;
    function CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean; virtual; abstract;
  	function SetProgRampEx(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize:integer;PRec:PRampEx):boolean;virtual; abstract;
	  function GetProgRampEx(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):boolean;virtual; abstract;
    //Thermo
    function SetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;virtual; abstract;
    function GetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;virtual; abstract;
    // HPLC
		function SetFlowRec(AZone,AIndex:integer;PRec:PFlowRec):boolean;virtual; deprecated;
    // samples
    function GetSampParam(ATower,AParamIndex:integer):integer;virtual;
  	function SetSampParam(ATower,AParamIndex,AParam:integer):boolean;virtual;

    function GetEventParam(AEvent,AParamIndex:integer):integer;virtual;
  	function SetEventParam(AEvent,AParamIndex,AParam:integer):boolean;virtual;

    function SetSample(ASamp,ATower,AVial,ACount,AVolume:integer):boolean;virtual;
  	function GetSample(ASamp,AParamIndex:integer):integer;virtual;


		// Update
		// update
		function Update(AParam:integer):boolean;virtual;


	{	function GetSampValue(ASamp,AValueIndex:integer):double;virtual;
		function SetSampValue(ASamp,AValueIndex:integer;AValue:double):BOOL;virtual;}
    // properties
    property Driver:TDriver read FDriver;
    property Profile:integer read FProfile;
    property Handle:IHandle read FHandle;
		property Action:TNotify read FAction;
		property Client:Pointer read FClient;
		property Frequency[AChannel:integer]:single read GetFrequency;
		property NeedExit:boolean read FNeedExit;
		property ChannelsSize:integer read FChannelsSize write SetChannelsSize;
		property State:integer read FState write IntSetState;
		property NewState:integer read FNewState;
		property IsSetState:boolean read FSetState write FSetState;
		property StateChangeTime:integer read FStateChangeTime;
		property StateTime:integer read GetStateTime;
		property DefaultFrequency:single read FDefaultFrequency write FDefaultFrequency;
	end;

	TDriverClass=class of TDriver;
	TInstrumentClass=class of TInstrument;

	function drvOpenEx(ADebugFunction:TDebugWriteStr;AClient:Pointer):BOOL;stdcall;
	function drvClose:BOOL;stdcall;
	function drvOpenInstrument(AProfile:integer;AnAction:TNotify;AClient:pointer):integer;stdcall;
	function drvCloseInstrument(HInstrument:integer):BOOL;stdcall;
	function drvOpenChannelEx(HInstrument:IHandle;AnAction:TInt32Callback;AClient:pointer;AChannel:integer):BOOL;stdcall;
	function drvCloseChannelEx(HInstrument:IHandle;AChannel:integer):BOOL;stdcall;
	function drvGetChannelInfo(HInstrument:IHandle;AChannel:integer;var DevInfo:TDeviceInfo):BOOL;stdcall;
	function drvGetChannelFrequency(HInstrument:IHandle;AChannel:integer):single;stdcall;
	function drvSetChannelFrequency(HInstrument:IHandle;AChannel:integer;freq:single):single;stdcall;
	function drvChannelCalibrate(HInstrument:IHandle;AChannel:integer;Adata:double):double;stdcall;
	function drvSetState(HInstrument:IHandle;AState:integer):BOOL;stdcall;
	function drvGetState(HInstrument:IHandle):integer;stdcall;
  // Unified GC/LC
	function drvGetZoneInfo(HInstrument:IHandle;AZone,AZoneObject:integer;lpBuf:PChar;var cb:Integer):boolean;stdcall;
	function tcbSetZoneValueEx(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer;AValue:extended):BOOL;stdcall;
	function tcbGetZoneValueEx(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):extended;stdcall;
	function tcbCheckReadZoneValue(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
	function tcbCheckWriteZoneValue(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
  function drvSetProgRampEx(HInstrument:IHandle;AZone,AZoneObject,ARampIndex,ARampSize:integer;PRec:PRampEx):BOOL;stdcall;
  function drvGetProgRampEx(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):BOOL;stdcall;
	//Thermo
	function tcbSetProgRamp(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;
	function tcbGetProgRamp(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;
	// hplc
	function fcbSetRecEx(hnd,zone,ndx:integer;Prec:PFlowRec):bool;stdcall; deprecated;
	// Sampler
	function smpGetParam(HInstrument:IHandle;ATower,AParamIndex:integer):integer;stdcall;
	function smpSetParam(HInstrument:IHandle;ATower,AParamIndex,AParam:integer):BOOL;stdcall;

	function smpSetSample(HInstrument:IHandle;ASamp,ATower,AVial,ACount,AVolume:integer):boolean;stdcall;
	function smpGetSample(HInstrument:IHandle;ASamp,AParamIndex:integer):integer;stdcall;

	function evtGetParam(HInstrument:IHandle;AEvent,AParamIndex:integer):integer;stdcall;
	function evtSetParam(HInstrument:IHandle;AEvent,AParamIndex,AParam:integer):BOOL;stdcall;

	// update
	function drvUpdate(HInstrument:IHandle;AParam:integer):boolean;stdcall;


var
	Driver:TDriver;

	DriverClass:TDriverClass;
	InstrumentClass:TInstrumentClass;

const
	sProfilePath='Software\New Analytical Systems\UniChrom\I';
	sStates:array[dsNormal..dsCooling]of string=
	(
		'NORMAL',
		'PRERUN',
		'START',
    'POSTRUN',
    'COOLING'
  );

  sZoneTypes:array[ztOven..ztDet] of string=
  (
    'Oven',
    'Inj',
    'Det'
  );
  sObjTypes:array[zotTemp..zotVel] of string=
  (
    'Temp',
    'Flow',
    'Pres',
    'Vel'
  );
  sParamTypes:array[opMin..opLight] of string=
  (
    'Min',
    'Max',
    'Set',
    'Cur',
    'Drift',
    'Gas',
    'Prog',
    'Name',
    'Light'
  );

  sBool:array[false..true] of string=
  (
    'FALSE',
    'TRUE'
  );

	function sCode(ind:integer):char;

	function ConvertIndices(AZone,AZoneObject{,AParamIndex}:integer; var AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer):boolean;

implementation
uses atick;


// additional debug functions

  function sCode(ind:integer):char;
  begin
    Result:=Char(ind+ord('A'));
  end;

// private internal functions

function ConvertIndices(AZone,AZoneObject{,AParamIndex}:integer; var AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer):boolean;
begin
  if AZone<0
  then begin
    AZoneType:=-1;
    AZoneIndex:=-1;
  end
  else begin
    AZoneType:=AZone shr zTypeShift;
    AZoneIndex:=AZone and zValueIndex;
  end;
  if AZoneObject<0
  then begin
    AObjectType:=-1;
    AObjectIndex:=-1;
  end
  else begin
    AObjectType:=AZoneObject shr zTypeShift;
    AObjectIndex:=AZoneObject and zValueIndex;
  end;
  Result:=true;
end;

function ProfilePath(AProfile:integer):string;
begin
  Result:=sProfilePath+IntToStr(AProfile);
end;

// all exported functions

function drvOpenEx(ADebugFunction:TDebugWriteStr;AClient:Pointer):BOOL;stdcall;
begin
  if not assigned(Driver) then
  try
    Driver:=DriverClass.Create;
  except
    Driver:=nil;
  end;
  Result:=Assigned(Driver);
  if Result then
  with Driver do
  begin
    DebugFunction:=ADebugFunction;
    DebugFunctionClient:=AClient;
  end;
end;

function drvClose:BOOL;stdcall;
begin
  if Assigned(Driver) then
  with Driver do
  begin
    Free;
    Driver:=nil;
    Result:=true;
  end else Result:=false;
end;

function drvOpenInstrument(AProfile:integer;AnAction:TNotify;AClient:pointer):integer;stdcall;
begin
  if Assigned(Driver) then result:=Driver.OpenInstrument(AProfile,AnAction,AClient)
  else result:=0;
end;

function drvCloseInstrument(HInstrument:integer):BOOL;stdcall;
begin
  if Assigned(Driver) then result:=Driver.CloseInstrument(HInstrument)
  else result:=false;
end;

function drvOpenChannelEx(HInstrument:IHandle;AnAction:TInt32Callback;AClient:pointer;AChannel:integer):BOOL;stdcall;
begin
  if Assigned(Driver) then result:=Driver.OpenChannel(HInstrument,AnAction,AClient,AChannel)
  else result:=false;
end;

function drvCloseChannelEx(HInstrument:IHandle;AChannel:integer):BOOL;stdcall;
begin
  if Assigned(Driver) then result:=Driver.CloseChannel(HInstrument,AChannel)
  else result:=false;
end;

function drvGetChannelInfo(HInstrument:IHandle;AChannel:integer;var DevInfo:TDeviceInfo):BOOL;stdcall;
begin
  if Assigned(Driver) then result:=Driver.GetChannelInfo(HInstrument,AChannel,DevInfo)
  else result:=false;
end;

function drvGetChannelFrequency(HInstrument:IHandle;AChannel:integer):single;stdcall;
begin
  if Assigned(Driver) then result:=Driver.GetFrequency(HInstrument,AChannel)
  else result:=0;
end;

function drvSetChannelFrequency(HInstrument:IHandle;AChannel:integer;freq:single):single;stdcall;
begin
  if Assigned(Driver) then result:=Driver.SetFrequency(HInstrument,AChannel,freq)
  else result:=0;
end;

function drvChannelCalibrate(HInstrument:IHandle;AChannel:integer;Adata:double):double;stdcall;
begin
  if Assigned(Driver) then Result:=Driver.Calibrate(HInstrument,AChannel,Adata)
  else result:=0;
end;

function drvSetState(HInstrument:IHandle;AState:integer):BOOL;stdcall;
begin
  if Assigned(Driver) then Result:=Driver.SetState(HInstrument,AState)
  else result:=false;
end;

function drvGetState(HInstrument:IHandle):integer;stdcall;
begin
  if Assigned(Driver) then Result:=Driver.GetState(HInstrument)
  else result:=-1;
end;

function drvGetZoneInfo(HInstrument:IHandle;AZone,AZoneObject:integer;lpBuf:PChar;var cb:Integer):boolean;stdcall;
var
	AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
	Result:=false;
	if Assigned(Driver) then
	if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
	Result:=Driver.GetZoneInfo(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,lpBuf,cb)
  else Result:=false;
end;

function tcbSetZoneValueEx(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer;AValue:extended):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  Result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.SetZoneValue(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex,AValue);
end;

function tcbGetZoneValueEx(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):extended;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=InvalidZoneValue;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.GetZoneValue(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function tcbCheckReadZoneValue(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.CheckReadZoneValue(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function tcbCheckWriteZoneValue(HInstrument:IHandle;AZone,AZoneObject,AParamIndex:integer):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.CheckWriteZoneValue(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function tcbSetProgRamp(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.SetProgRamp(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,PRec);
end;

function tcbGetProgRamp(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;PRec:PRamp):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.GetProgRamp(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,PRec);
end;

function drvSetProgRampEx(HInstrument:IHandle;AZone,AZoneObject,ARampIndex,ARampSize:integer;PRec:PRampEx):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.SetProgRampEx(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize,PRec);
end;

function drvGetProgRampEx(HInstrument:IHandle;AZone,AZoneObject,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):BOOL;stdcall;
var
  AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;
begin
  result:=false;
  if Assigned(Driver) then
  if ConvertIndices(AZone,AZoneObject,AZoneType,AZoneIndex,AObjectType,AObjectIndex) then
  Result:=Driver.GetProgRampEx(HInstrument,AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize,PRec);
end;

// sample routines
function smpGetParam(HInstrument:IHandle;ATower,AParamIndex:integer):integer;stdcall;
begin
	Result:=InvalidZoneValue;
  if Assigned(Driver) then
  Result:=Driver.GetSampParam(HInstrument,ATower,AParamIndex);
end;

function smpSetParam(HInstrument:IHandle;ATower,AParamIndex,AParam:integer):BOOL;stdcall;
begin
	Result:=False;
  if Assigned(Driver) then
  Result:=Driver.SetSampParam(HInstrument,ATower,AParamIndex,AParam);
end;

function evtGetParam(HInstrument:IHandle;AEvent,AParamIndex:integer):integer;stdcall;
begin
	Result:=InvalidZoneValue;
  if Assigned(Driver) then
  Result:=Driver.GetEventParam(HInstrument,AEvent,AParamIndex);
end;

function evtSetParam(HInstrument:IHandle;AEvent,AParamIndex,AParam:integer):BOOL;stdcall;
begin
	Result:=False;
	if Assigned(Driver) then
	Result:=Driver.SetEventParam(HInstrument,AEvent,AParamIndex,AParam);
end;

function smpGetSample(HInstrument:IHandle;ASamp,AParamIndex:integer):integer;stdcall;
begin
	Result:=InvalidZoneValue;
	if Assigned(Driver) then
	Result:=Driver.GetSample(HInstrument,ASamp,AParamIndex);
end;

function smpSetSample(HInstrument:IHandle;ASamp,ATower,AVial,ACount,AVolume:integer):boolean;stdcall;
begin
	Result:=False;
	if Assigned(Driver) then
	Result:=Driver.SetSample(HInstrument,ASamp,ATower,AVial,ACount,AVolume);
end;

function fcbSetRecEx(hnd,zone,ndx:integer;Prec:PFlowRec):bool;
begin
	Result:=False;
	if Assigned(Driver) then
	Result:=Driver.SetFlowRec(hnd,zone,ndx,prec);
end;

function drvUpdate(HInstrument:IHandle;AParam:integer):boolean;
begin
	Result:=False;
	if Assigned(Driver) then
	Result:=Driver.Update(HInstrument,AParam);
end;

// TDriver

constructor TDriver.Create;
var
	buf:array[0..1024] of char;
	l:dword;
begin
	l:=GetModuleFileName(HInstance,buf,sizeof(buf));
	if l=0 then FModuleName:='unknown.dll' else SetString(FModuleName,buf,l);
end;

destructor TDriver.Destroy;
begin
  SetInstrumentsSize(0);
end;

{$J+}
function TDriver.GetUniqueHandle:IHandle;
const
  newid:IHandle=0;
begin
  repeat
    inc(newid);
  until (newid<>0)and(FindInstrumentByHandle(newid)=-1);
  Result:=newid;
end;
{$J-}

function TDriver.FindInstrumentByHandle(AHandle:IHandle):integer;
var
  i:integer;
begin
  result:=-1;
  for i:=0 to InstrumentsSize-1 do
  if assigned(FInstruments[i]) then
  if FInstruments[i].Handle=AHandle then
  begin
    result:=i;
    break;
  end;
end;

function TDriver.GetInstruments(index:integer):TInstrument;
begin
  Result:=nil;
	if (index<0) or (index>=InstrumentsSize) then exit;
  Result:=fInstruments[index];
end;

procedure TDriver.SetInstrumentsSize(AInstrumentsSize:integer);
var
  i:integer;
begin
  if AInstrumentsSize<FInstrumentsSize
  then for i:=AInstrumentsSize to FInstrumentsSize-1 do FInstruments[i].Free;
  ReallocMem(FInstruments,AInstrumentsSize*sizeof(FInstruments[0]));
  if AInstrumentsSize>FInstrumentsSize
  then for i:=FInstrumentsSize to AInstrumentsSize-1 do FInstruments[i]:=nil;
  FInstrumentsSize:=AInstrumentsSize;
end;

procedure TDriver.DbgWrite(AMessage:String);
begin
  if assigned(DebugFunction) then
  begin
    DebugFunction(DebugFunctionClient,Pointer(DeviceName+': '+AMessage));
  end else
  begin
    WriteMessage(AMessage,DeviceName,'',true);
    //Writeln(AMessage);
  end;
end;

procedure TDriver.CloseAll;
var
  i:integer;
begin
  for i:=0 to InstrumentsSize-1 do
  begin
    FInstruments[i].Free;
    FInstruments[i]:=nil;
  end;
end;

function TDriver.OpenInstrument(AProfile:integer;AnAction:TNotify;AClient:pointer):IHandle;
begin
  Result:=0;
  if (AProfile<0) then exit;
  if (AProfile>=InstrumentsSize) then InstrumentsSize:=AProfile+1;
  if assigned(FInstruments[AProfile]) then exit;
  try
    FInstruments[AProfile]:=InstrumentClass.Create(Self, AProfile, AnAction, AClient);
    result:=FInstruments[AProfile].Handle;
    if result=0 then
    begin
      FInstruments[AProfile].Free;
      FInstruments[AProfile]:=nil;
    end;
	except
		on E:Exception do
		begin
			FInstruments[AProfile]:=nil;
			DbgWrite('Error in opening instrument:'+E.Message);
		end;
  end;
end;

function TDriver.CloseInstrument(HInstrument:IHandle):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  try
    FInstruments[AProfile].Free;
    result:=true;
  except
    DbgWrite('Error in closing instrument');
  end;
  FInstruments[AProfile]:=nil;
end;

function TDriver.OpenChannel(HInstrument:IHandle;AnAction:TInt32Callback;AClient:pointer;AChannel:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].OpenChannel(AnAction,AClient,AChannel);
end;

function TDriver.CloseChannel(HInstrument:IHandle;AChannel:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].CloseChannel(AChannel);
end;

function TDriver.GetChannelInfo(HInstrument:IHandle;AChannel:integer;var DevInfo:TDeviceInfo):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].GetChannelInfo(AChannel,DevInfo);
end;

function TDriver.GetFrequency(HInstrument:IHandle;AChannel:integer):single;
var
  AProfile:integer;
begin
  result:=0;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].GetFrequency(AChannel);
end;

function TDriver.SetFrequency(HInstrument:IHandle;AChannel:integer;freq:single):single;
var
  AProfile:integer;
begin
  result:=0;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].SetFrequency(AChannel,freq);
end;

function TDriver.Calibrate(HInstrument:integer;AChannel:integer;Adata:double):double;
var
  AProfile:integer;
begin
  result:=AData;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].Calibrate(AChannel,AData);
end;

function TDriver.GetState(HInstrument:IHandle):integer;
var
  AProfile:integer;
begin
  result:=-1;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].GetState;
end;

function TDriver.SetState(HInstrument:IHandle;AState:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].SetState(AState);
end;

function TDriver.GetZoneInfo(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex:integer;lpBuf:PChar;var cb:Integer):boolean;
var
	AProfile:integer;
begin
	result:=false;
	AProfile:=FindInstrumentByHandle(HInstrument);
	if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
	result:=FInstruments[AProfile].GetZoneInfo(AZoneType,AZoneIndex,AObjectType,AObjectIndex,lpBuf,cb);
end;

function TDriver.Update(HInstrument:IHandle;AParam:integer):boolean;
var
	AProfile:integer;
begin
	result:=false;
	AProfile:=FindInstrumentByHandle(HInstrument);
	if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
	result:=FInstruments[AProfile].Update(AParam);
end;


//
function TDriver.SetZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer;AValue:extended):boolean;
var
	AProfile:integer;
begin
	result:=false;
	AProfile:=FindInstrumentByHandle(HInstrument);
	if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
	if FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex)
	then result:=FInstruments[AProfile].SetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex,AValue);
end;

function TDriver.GetZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):extended;
var
	AProfile:integer;
begin
	result:=InvalidZoneValue;
	AProfile:=FindInstrumentByHandle(HInstrument);
	if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
	if FInstruments[AProfile].CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex)
	then result:=FInstruments[AProfile].GetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function TDriver.CheckReadZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function TDriver.CheckWriteZoneValue(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  result:=FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
end;

function TDriver.SetProgRampEx(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize:integer;PRec:PRampEx):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  if FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,opSet)
  then result:=FInstruments[AProfile].SetProgRampEx(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize,PRec);
end;

function TDriver.GetProgRampEx(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;var ARampSize:integer; PRec:PRampEx):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  if FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,opSet)
  then result:=FInstruments[AProfile].GetProgRampEx(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,ARampSize,PRec);
end;

function TDriver.SetProgRamp(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  if FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,opSet)
  then result:=FInstruments[AProfile].SetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,PRec);
end;

function TDriver.GetProgRamp(HInstrument:IHandle;AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  if FInstruments[AProfile].CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,opSet)
  then result:=FInstruments[AProfile].GetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex,PRec);
end;

// samples
function TDriver.GetSampParam(HInstrument:IHandle;ATower,AParamIndex:integer):integer;
var
  AProfile:integer;
begin
  result:=InvalidZoneValue;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].GetSampParam(ATower,AParamIndex);
end;

function TDriver.SetSampParam(HInstrument:IHandle;ATower,AParamIndex,AParam:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].SetSampParam(ATower,AParamIndex,AParam);
end;

function TDriver.GetEventParam(HInstrument:IHandle;AEvent,AParamIndex:integer):integer;
var
  AProfile:integer;
begin
  result:=InvalidZoneValue;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].GetEventParam(AEvent,AParamIndex);
end;

function TDriver.SetEventParam(HInstrument:IHandle;AEvent,AParamIndex,AParam:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].SetEventParam(AEvent,AParamIndex,AParam);
end;

function TDriver.GetSample(HInstrument:IHandle;ASamp,AParamIndex:integer):integer;
var
  AProfile:integer;
begin
  result:=InvalidZoneValue;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].GetSample(ASamp,AParamIndex);
end;

function TDriver.SetSample(HInstrument:IHandle;ASamp,ATower,AVial,ACount,AVolume:integer):boolean;
var
  AProfile:integer;
begin
  result:=false;
  AProfile:=FindInstrumentByHandle(HInstrument);
  if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
  Result:=FInstruments[AProfile].SetSample(ASamp,ATower,AVial,ACount,AVolume);
end;

function TDriver.SetFlowRec(HInstrument:IHandle;Azone,AIndex:integer;Prec:PFlowRec):boolean; deprecated;
var
	AProfile:integer;
begin
	result:=false;
	AProfile:=FindInstrumentByHandle(HInstrument);
	if (AProfile<0) or (AProfile>=InstrumentsSize) then exit;
	Result:=FInstruments[AProfile].SetFlowRec(AZone,AIndex,PRec);
end;




// TInstrument

constructor TInstrument.Create(ADriver:TDriver; AProfile:integer;AnAction:TNotify;AClient:pointer);
begin
  FDriver:=ADriver;
  FProfile:=AProfile;
  FAction:=AnAction;
  FClient:=AClient;
  DbgWrite('Created');
  FDefaultFrequency:=25;
  if OpenDevice
  then begin
    if OpenThread then FHandle:=ADriver.GetUniqueHandle
    else CloseDevice;
  end;
end;

destructor TInstrument.Destroy;
begin
  CloseThread;
  CloseDevice;
  SetChannelsSize(0);
  DbgWrite('Freed');
end;

procedure TInstrument.DbgWrite(AMessage:string);
var
  s:string;
begin
  s:='I'+IntToStr(FProfile)+': '+AMessage;
  if assigned(Driver) then
  begin
    Driver.DbgWrite(s);
  end else
  begin
    WriteMessage(s,'','',true);
  end;
end;

function TInstrument.ProfilePath:string;
begin
  Result:=ad2001.ProfilePath(FProfile);
end;

procedure TInstrument.SetChannelsSize(AChannelsSize:integer);
var
  i:integer;
begin
  ReallocMem(FChannels,AChannelsSize*sizeof(FChannels[0]));
  if AChannelsSize>FChannelsSize
  then for i:=FChannelsSize to AChannelsSize-1
  do with FChannels[i] do
  begin
    Action:=nil;
    Frequency:=DefaultFrequency;
  end;
  FChannelsSize:=AChannelsSize;
end;

function TInstrument.OpenDevice:boolean;
begin
  ChannelsSize:=15;
  result:=true;
end;

function TInstrument.OpenThread:boolean;
begin
  result:=FThread<>0;
  if result then exit;
  FNeedExit:=false;
  {$IFDEF MSWINDOWS}
  FThread:=BeginThread(nil,8192,@TInstrument.Work,Self,0,FThreadID);
  result:=FThread<>0;
  if result then SetThreadPriority(FThread,THREAD_PRIORITY_TIME_CRITICAL);
  {$ENDIF}
  {$IFDEF LINUX}
  FThread:=ord(BeginThread(nil,@TInstrument.Work,Self,FThreadID)=0);
  result:=FThread<>0;
  {$ENDIF}
end;

procedure TInstrument.CloseDevice;
begin
end;

procedure TInstrument.CloseThread;
var res:cardinal;
begin
  if FThread<>0 then
  begin
    FNeedExit:=true;
    {$IFDEF MSWINDOWS}
    if WaitForSingleObject(FThread,1000)=WAIT_TIMEOUT
    then begin
      TerminateThread(FThread,0);
      MessageBeep(UINT(-1));
      DbgWrite('Thread was terminated abnormally');
    end;
    Closehandle(FThread);
    {$ENDIF}
    {$IFDEF LINUX}
    if FThreadID <> 0 then
    begin
      pthread_join(fThreadId,@res);
      pthread_detach(FThreadID);
    end;
    {$ENDIF}
    FThread:=0;
  end;
end;

function TInstrument.Work:integer;
var
  Buffer: array[0..1023] of Char;
begin
  repeat
    try
      Result:=WorkThread;
    except
      //ShowException(ExceptObject, ExceptAddr);
      ExceptionErrorMessage(ExceptObject, ExceptAddr, Buffer, SizeOf(Buffer));
      DbgWrite(Buffer);
      DbgWrite('Thread was terminated with exception, restarting...');
      Result:=0;
      if not NeedExit then Sleep(100);
    end;
  until NeedExit;
end;

function TInstrument.OpenChannel(AnAction:TInt32Callback;AClient:pointer;AChannel:integer):boolean;
begin
  Result:=false;
  if (AChannel<0) or (AChannel>=ChannelsSize) then exit;
  with FChannels[AChannel]do
  begin
    if Assigned(Action)then exit;
    Action:=AnAction;
    Client:=AClient;
  end;
  Result:=true;
end;

function TInstrument.CloseChannel(AChannel:integer):boolean;
begin
  Result:=false;
  if (AChannel<0) or (AChannel>=ChannelsSize) then exit;
  with FChannels[AChannel]do
  begin
    if not assigned(Action) then exit;
    Action:=nil;
    Client:=nil;
  end;
  Result:=true;
end;

function TInstrument.IsActiveChannel(AChannel:integer):boolean;
begin
  Result:=assigned(FChannels) and assigned(FChannels[AChannel].Action);
end;

procedure TInstrument.IntSetState(AState:integer);
begin
  FState:=AState;
  FStateChangeTime:=systicks();
end;

function TInstrument.GetStateTime:integer;
begin
  Result:=systicks-StateChangeTime;
end;

function TInstrument.GetFrequency(AChannel:integer):single;
begin
  Result:=0;
  if (AChannel<0) or (AChannel>=ChannelsSize) then exit;
  Result:=FChannels[AChannel].Frequency;
end;

function TInstrument.SetFrequency(AChannel:integer;freq:single):single;
begin
  Result:=0;
  if (AChannel<0) or (AChannel>=ChannelsSize) then exit;
  FChannels[AChannel].Frequency:=freq;
  Result:=freq;
end;

function TInstrument.Calibrate(AChannel:integer;Adata:double):double;
begin
  Result:=Adata;
end;

function TInstrument.GetState:integer;
begin
  Result:=FState;
end;

function TInstrument.SetState(AState:integer):boolean;
begin
  FNewState:=AState;
  FSetState:=true;
  result:=true;
end;

procedure TInstrument.CallPointCallback(AChannel:integer;data:integer;stat:integer);
begin
  try
    if IsActiveChannel(AChannel) then TPointCallback(FChannels[AChannel].Action)(FChannels[AChannel].Client,data,stat);
  except
    DbgWrite('Exception while calling TPointCallback');
  end;
end;

procedure TInstrument.CallBlockCallback(AChannel:integer;PBuf:PIntArray;cbLen:integer);
begin
  try
    if IsActiveChannel(AChannel) then TBlockCallback(FChannels[AChannel].Action)(FChannels[AChannel].Client,PBuf,cbLen);
  except
    DbgWrite('Exception while calling TBlockCallback');
  end;
end;

procedure TInstrument.CallInt32Callback(AChannel:integer;PData:PDwordArray;cbLen:integer;stat:integer);
begin
  try
    if IsActiveChannel(AChannel) then TInt32Callback(FChannels[AChannel].Action)(FChannels[AChannel].Client,PData,cbLen,stat);
  except
    DbgWrite('Exception while calling TInt32Callback');
  end;
end;

procedure TInstrument.CallInt64Callback(AChannel:integer;PData:PQwordArray;cbLen:integer;stat:integer);
begin
  try
  if IsActiveChannel(AChannel) then TInt64Callback(FChannels[AChannel].Action)(FChannels[AChannel].Client,PData,cbLen,stat);
  except
    DbgWrite('Exception while calling TInt64Callback');
  end;
end;

procedure TInstrument.CallFloatCallback(AChannel:integer;PData:PFloatArray;cbLen:integer;stat:integer);
begin
  try
  if IsActiveChannel(AChannel) then TFloatCallback(FChannels[AChannel].Action)(FChannels[AChannel].Client,PData,cbLen,stat);
  except
    DbgWrite('Exception while calling TFloatCallback');
  end;
end;

procedure TInstrument.CallDoubleCallback(AChannel:integer;PData:PDoubleArray;cbLen:integer;stat:integer);
begin
  try
    if IsActiveChannel(AChannel) then TDoubleCallback(FChannels[AChannel].Action)(FChannels[AChannel].Client,PData,cbLen,stat);
  except
    DbgWrite('Exception while calling TDoubleCallback');
  end;
end;

procedure TInstrument.CallNotify(AZone:integer);
begin
  try
    if assigned(Action) then Action(Client,AZone,Handle);
  except
    DbgWrite('Exception while calling TNotify');
  end;
end;

function TInstrument.GetSampParam(ATower,AParamIndex:integer):integer;
begin
	Result:=InvalidZoneValue;
end;

function TInstrument.SetSampParam(ATower,AParamIndex,AParam:integer):boolean;
begin
	Result:=false;
end;

function TInstrument.GetEventParam(AEvent,AParamIndex:integer):integer;
begin
	Result:=InvalidZoneValue;
end;

function TInstrument.SetEventParam(AEvent,AParamIndex,AParam:integer):boolean;
begin
	Result:=false;
end;

function TInstrument.GetSample(ASamp,AParamIndex:integer):integer;
begin
	Result:=InvalidZoneValue;
end;

function TInstrument.SetSample(ASamp,ATower,AVial,ACount,AVolume:integer):boolean;
begin
	Result:=false;
end;

function TInstrument.SetFlowRec(AZone,AIndex:integer;PRec:PFlowRec):boolean; deprecated;
begin
	Result:=false;
end;

function TInstrument.Update(AParam:integer):boolean;
begin
	result:=true;
end;

end.