unit drv_common;

interface

{$H+}

uses Types,SysUtils,
     {$IFDEF MSWINDOWS}Windows{$ENDIF}
     {$IFDEF LINUX}Libc,IniFiles,strtools{$ENDIF};

{$IFDEF LINUX}
type
  TIniReg=class(TIniFile)
  public
    defSection:string;
    dwAccess:dword;
  end;

  HKEY=THandle;  // under linux HKEY is a pointer to TIniReg object



const
  HKEY_LOCAL_MACHINE=THandle(1);
  HKEY_CURRENT_USER=THandle(2);

  KEY_READ=1;
  KEY_WRITE=2;

  REG_OPTION_NON_VOLATILE=0;
  REG_SZ=1;


  ERROR_SUCCESS=0;
  ERROR_PROBLEM=-1;

{$ENDIF}

procedure WriteMessage(Msg,DevName,IntfName:string;WriteTime:boolean);

{$IFDEF LINUX} // simulate registry behavior
function RegCreateKeyEx(RootKey: HKEY; lpSubKey: PChar;
  Reserved: DWORD; lpClass: PChar; dwOptions: DWORD; samDesired: DWORD;
  lpSecurityAttributes: pointer; var phkResult: HKEY;
  lpdwDisposition: PDWORD): Longint;

function RegOpenKeyEx(RootKey: HKEY; lpSubKey: PChar;
  ulOptions: DWORD; samDesired: DWORD; var phkResult: HKEY): Longint;

function RegClosekey(Key:HKEY):integer;
{$ENDIF}
function RegLoadStringDef(RegKey:HKEY;KeyName:string;DefValue:string):string;
function RegLoadIntegerDef(RegKey:HKEY;KeyName:string;DefValue:integer):integer;
function RegLoadRealDef(RegKey:HKEY;KeyName:string;DefValue:extended):extended;
function RegLoadBoolDef(RegKey:HKEY;KeyName:string;DefValue:boolean):boolean;


function IntToStr(I:Integer):String;
function IntToStrF(I:Integer;W:integer):String;
function DigitToChar(B:byte):char;
function ByteToHex(b:byte):String;
function BufToHex(var Buf;cb:integer):string;
function BufDump(var Buf;cb:integer;NPSReplace:char='.'):string; overload;
function BufDump(s:string;NPSReplace:char='.'):string; overload;
function BufAsText(var Buf;cb:integer;NPSReplace:char='.'):string; overload;
function BufAsText(s:string;NPSReplace:char='.'):string; overload;

implementation

{$I-}
procedure WriteMessage(Msg,DevName,IntfName:string;WriteTime:boolean);
var
//  ST:TSystemTime;
  s:string;
begin
	if Msg='' then exit;
	try
		s:='';
		if WriteTime then
		begin
			{GetLocalTime(ST);
			with ST do s:=Format('%.2u.%.2u %.2u:%.2u:%.2u.%.3u ',[wDay,wMonth,wHour,wMinute,wSecond,wMilliseconds]);}
      s:=FormatDateTime('dd.mm.hh.nn.ss.zzz',Now())+#9;
		end;
		if (DevName<>'')and(IntfName<>'') then s:=s+DevName+' on '+IntfName+' '
		else if (DevName<>'')or(IntfName<>'') then s:=s+DevName+IntfName+' ';
		s:=s+Msg;
	//	Flush(Output);
		writeln(s);
		Flush(Output);
  except
    if IsConsole
    then begin
      Writeln('WriteMessage()');
      Flush(Output);
      SysUtils.ShowException(ExceptObject, ExceptAddr);
    end;
  end;
end;

{$IFDEF MSWINDOWS}
function RegLoadStringDef(RegKey:HKEY;KeyName:string;DefValue:string):string;
var
  RegType,cb:integer;
  data:array[0..1023] of char;
begin
  Result:=DefValue;
  cb:=sizeof(data);
  if RegQueryValueEx(RegKey,PChar(KeyName),nil,@RegType,@data,@cb)<>ERROR_SUCCESS then cb:=0;
  if cb<>0 then
  case RegType of
    REG_BINARY:
    begin
      SetString(Result,data,cb);
    end;
    REG_SZ:
    begin
      Result:=data;
    end;
  end;
end;

function RegLoadIntegerDef(RegKey:HKEY;KeyName:string;DefValue:integer):integer;
var
  RegType,cb:integer;
  data:array[0..128] of char;
  intdata:integer absolute data;
  err:integer;
begin
  Result:=DefValue;
  cb:=sizeof(data);
  if RegQueryValueEx(RegKey,Pointer(KeyName),nil,@RegType,@data,@cb)<>ERROR_SUCCESS then cb:=0;
  if cb<>0 then
  case RegType of
    REG_BINARY,REG_DWORD: if cb=sizeof(intdata) then Result:=intdata;
    REG_SZ:
    begin
      val(data,Result,err);
      if err<>0 then Result:=DefValue;
    end;
  end;
end;

function RegLoadBoolDef(RegKey:HKEY;KeyName:string;DefValue:boolean):boolean;
var
  RegType,cb:integer;
  data:array[0..128] of char;
  bytedata:byte absolute data;
  worddata:word absolute data;
  intdata:integer absolute data;
  i,err:integer;
begin
  Result:=DefValue;
  cb:=sizeof(data);
  if RegQueryValueEx(RegKey,Pointer(KeyName),nil,@RegType,@data,@cb)<>ERROR_SUCCESS then cb:=0;
  if cb<>0 then
  case RegType of
    REG_BINARY,REG_DWORD:
    case cb of
      sizeof(bytedata): Result:=bytedata<>0;
      sizeof(worddata): Result:=worddata<>0;
      sizeof(intdata): Result:=intdata<>0;
    end;
    REG_SZ:
    begin
      val(data,i,err);
      if err=0 then Result:=i<>0
      else begin
        case UpCase(data[0]) of
          'Y','T': Result:=true;
          'N','F': Result:=false;
        end;
      end;
    end;
  end;
end;


function RegLoadRealDef(RegKey:HKEY;KeyName:string;DefValue:extended):extended;
var
  RegType,cb,i:integer;
  data:array[0..128] of char;
  intdata:integer absolute data;
  singledata:single absolute data;
  doubledata:double absolute data;
  extendeddata:extended absolute data;
  err:integer;
begin
  Result:=DefValue;
  cb:=sizeof(data);
  if RegQueryValueEx(RegKey,Pointer(KeyName),nil,@RegType,@data,@cb)<>ERROR_SUCCESS then cb:=0;
  if cb<>0 then
  case RegType of
    REG_DWORD: if cb=sizeof(intdata) then Result:=intdata;
    REG_BINARY:
    case cb of
      sizeof(singledata): Result:=singledata;
      sizeof(doubledata): Result:=doubledata;
      sizeof(extendeddata): Result:=extendeddata;
    end;
    REG_SZ:
    begin
      for i:=0 to cb-1 do if data[i]=',' then data[i]:='.';
      val(data,Result,err);
      if err<>0 then Result:=DefValue;
    end;
  end;
end;
{$ENDIF}

{$IFDEF LINUX} // registry behavior through INI-files
function RegCreateKeyEx(RootKey: HKEY; lpSubKey: PChar;
  Reserved: DWORD; lpClass: PChar; dwOptions: DWORD; samDesired: DWORD;
  lpSecurityAttributes: pointer; var phkResult: HKEY;
  lpdwDisposition: PDWORD): Longint;
var ri:TIniReg;
    bs,soft,company,path,fname,section:string;
    p:integer;
begin
  Result:=-1;
  if (RootKey<>HKEY_LOCAL_MACHINE) and (RootKey<>HKEY_CURRENT_USER) then exit;
  case RootKey of
  HKEY_LOCAL_MACHINE:path:='/etc/unichrom/';
  HKEY_CURRENT_USER:path:=getenv('HOME')+'/.unichrom/';
  else
    path:='./';
  end;
  bs:=lpSubKey;
  soft:=NextToken(bs,'\');
  company:=NextToken(bs,'\');
  fname:=NextToken(bs,'\');
  section:=NextToken(bs,'\');

  if section='' then section:='@';

  CreateDir(path);
  ri:=TIniReg.Create(path+ChangeFileExt(fname,'.ini'));
  ri.defSection:=section;
  ri.dwAccess:=samDesired;
  phkResult:=hKey(ri);
  Result:=ERROR_SUCCESS;
end;

function RegOpenKeyEx(RootKey: HKEY; lpSubKey: PChar;
  ulOptions: DWORD; samDesired: DWORD; var phkResult: HKEY): Longint;
var desp:dword;
begin
  Result:=RegCreateKeyEx(RootKey,lpSubKey,0,nil,ulOptions,0,nil,phkResult,@desp);
end;


function RegClosekey(Key:HKEY):integer;
begin
  Result:=-1;
  if (Key=HKEY_LOCAL_MACHINE) or (Key=HKEY_CURRENT_USER) then exit;
  if (Key<>0) and (Tobject(Key) is TIniReg) then
  TObject(key).Free;
  Result:=ERROR_SUCCESS;
end;

function RegLoadStringDef(RegKey:HKEY;KeyName:string;DefValue:string):string;
begin
  Result:=DefValue;

  if (RegKey=0) or not (TObject(RegKey) is TIniReg) then exit;
  Result:=TIniReg(RegKey).ReadString(TiniReg(RegKey).defSection,KeyName,DefValue);

end;

function RegLoadIntegerDef(RegKey:HKEY;KeyName:string;DefValue:integer):integer;
begin
  Result:=DefValue;

  if (RegKey=0) or not (TObject(RegKey) is TIniReg) then exit;
  Result:=TIniReg(RegKey).ReadInteger(TiniReg(RegKey).defSection,KeyName,DefValue);
end;

function RegLoadRealDef(RegKey:HKEY;KeyName:string;DefValue:extended):extended;
begin
  Result:=DefValue;

  if (RegKey=0) or not (TObject(RegKey) is TIniReg) then exit;
  Result:=TIniReg(RegKey).ReadFloat(TiniReg(RegKey).defSection,KeyName,DefValue);
end;

function RegLoadBoolDef(RegKey:HKEY;KeyName:string;DefValue:boolean):boolean;
begin
  Result:=DefValue;

  if (RegKey=0) or not (TObject(RegKey) is TIniReg) then exit;
  Result:=TIniReg(RegKey).ReadBool(TiniReg(RegKey).defSection,KeyName,DefValue);
end;
{$ENDIF}


function IntToStr(I:Integer):String;
begin
  Str(I,Result);
end;

function IntToStrF(I:Integer;W:integer):String;
begin
  Str(I:W,Result);
end;

function DigitToChar(B:byte):char;
begin
  if B>9 then inc(B,7);
  Result:=chr(B+ord('0'));
end;

function ByteToHex(b:byte):String;
begin
  SetLength(Result,2);
  Result[1]:=DigitToChar(b shr 4);
  Result[2]:=DigitToChar(b and $0f);
end;

function BufToHex(var Buf;cb:integer):string;
var
  db:array[0..255] of byte absolute Buf;
  i:integer;
begin
  Result:=IntToStrF(cb,2)+':';
  for i:=0 to cb-1
  do begin
    Result:=Result+' '+ByteToHex(db[i]);
  end;
end;

function BufDump(var Buf;cb:integer;NPSReplace:char='.'):string;
var
  db:array[0..255] of byte absolute Buf;
  i:integer;
  sh,sc:string;
  c:char;
begin
  sh:='';
  sc:='';
  for i:=0 to cb-1
  do begin
    sh:=sh+' '+ByteToHex(db[i]);
    c:=char(db[i]);
    if (c<#$20)and(NPSReplace>=#$20) then c:=NPSReplace;
    sc:=sc+c;
  end;
  Result:=IntToStrF(cb,2)+':'+sh+' ('+sc+')';
end;

function BufDump(s:string;NPSReplace:char='.'):string;
begin
  Result:=BufDump(Pointer(@s[1])^,length(s),NPSReplace);
end;

function BufAsText(var Buf;cb:integer;NPSReplace:char='.'):string;
var
  db:array[0..255] of byte absolute Buf;
  i:integer;
begin
  SetString(Result,PChar(@Buf),cb);
  for i:=1 to cb
  do begin
    if (Result[i]<#$20)and(NPSReplace>=#$20) then Result[i]:=NPSReplace;
  end;
end;

function BufAsText(s:string;NPSReplace:char='.'):string;
begin
  Result:=BufAsText(Pointer(@s[1])^,length(s),NPSReplace);
end;

end.
