unit Main;

interface

{$J+}

uses
  ad2001,devtypes,sysutils,types,
  {$IFDEF MSWINDOWS}
  Windows
  {$ENDIF}
  {$IFDEF LINUX}
  Libc
  {$ENDIF};

type

  PProgram=^TProgram;
  TProgram=array[0..15]of TRamp;

  TGaussPeak=record
    A,P,S:double;
  end;

  TGCStubDriver=class(TDriver)
  protected
    function GetDeviceName:string; override;
  end;

  TGCStubInstrument=class(TInstrument)
  private
    fValues,fNewValues:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2,opMin..opLight] of double;
    //fValues,fNewValues:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2,opMax..opCur] of double;
    fSetValues:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2] of set of opMin..opLight;
    fPrograms,fNewPrograms:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2] of TProgram;
    fProgramsLength,fNewProgramsLength:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2] of Integer;
    fSetPrograms:array[ztOven..ztDet,0..3,zotTemp..zotVel,0..2] of boolean;
    fDataCounter:integer;
    fPeaks:array[0..15,0..14] of TGaussPeak;
    fBaseLines:array[0..15] of double;
    fNextPeakPos:array[0..15] of double;
    procedure ReInitPeaks(ch:integer);
    procedure NextPeak(ch,n:integer);
    procedure CheckPeaks(ch:integer);
    function CalcPeaks(ch:integer;t:double):double;
  protected
    function OpenDevice:boolean; override;
    function WorkThread:integer; override;
  public
    function SetFrequency(AChannel:integer;freq:single):single;override;
    function GetChannelInfo(AChannel:integer;var DevInfo:TDeviceInfo):boolean;override;
    //
    function SetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer;AValue:extended):boolean;override;
    function GetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):extended;override;
    function CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;override;
    function CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;override;
    function SetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;override;
    function GetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;override;
  end;

implementation
uses atick;

function TGCStubDriver.GetDeviceName;
begin
  Result:='GCStub';
end;

function GetProgramValue(var Prog:TProgram;ProgLen:integer;time:double;var Value:double):boolean;
var
  i:integer;
  v,t:double;
begin
  result:=false;
  if ProgLen=0 then exit;
  v:=0;
  for i:=0 to ProgLen-1 do
  begin
    if time<=0 then break;
    if Prog[i].ValRate<>0
    then t:=(Prog[i].FinalVal-v)/Prog[i].ValRate
    else t:=0;
    if t<0 then t:=0;
    if time<t
    then begin
      v:=v+Prog[i].ValRate*time;
    end
    else v:=Prog[i].FinalVal;
    time:=time-t-Prog[i].IsoTime;
  end;
  Value := v;
  result := time<=0;
end;

function TGCStubInstrument.OpenDevice:boolean;
var
  zt,zi,ot,oi:integer;
begin
  for zt:=Low(fValues) to High(fValues) do
  for zi:=Low(fValues[zt]) to High(fValues[zt]) do
  for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
  for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
  begin
    fValues[zt,zi,ot,oi,opMax]:=100+Random(400);
    fValues[zt,zi,ot,oi,opMin]:=-10+Random(20);
  end;
  ChannelsSize:=16;
  result:=true;
end;

function Gauss(A,T,T0,S:double):double;
begin
  Result:=A*exp(-sqr((T-T0)/S));
end;

function NGaussNoise:double;
const
  isSecond:boolean=false;
  SecondVal:double=0;
var
  _fi:double;
  _r:double;
begin
  if isSecond
  then begin
    isSecond:=false;
    Result:=SecondVal;
  end
  else begin
    _fi:=2*pi*random;
    _r:=sqrt(-2*ln(1-random));
    isSecond:=true;
    SecondVal:=_r*sin(_fi);
    Result:=_r*cos(_fi);
  end;
end;

function NExpoNoise:double;
begin
  Result:=-ln(1-random);
end;

function ST(T,T0,S:double):double;
begin
  Result:=(T-T0)/S;
  Result:=1/(1+sqr(Result))+Result;
  Result:=Result*S+T0;
end;

function SGauss(A,T,T0,S:double):double;
begin
  Result:=Gauss(A,ST(T,T0,3*S),T0,S);
end;

procedure TGCStubInstrument.ReInitPeaks(ch:integer);
var
  i:integer;
  P0:double;
begin
  P0:=30+random;
  for i:=Low(fPeaks[ch]) to High(fPeaks[ch])
  do with fPeaks[ch,i]
  do begin
    A:=random;
    P:=P0;
    P0:=P0+15*NExpoNoise{+5*random};
    S:=1+3*random;
  end;
  fNextPeakPos[ch]:=P0;
end;

procedure TGCStubInstrument.CheckPeaks(ch:integer);
var
  i:integer;
begin
  for i:=Low(fPeaks[ch]) to High(fPeaks[ch])
  do if (fPeaks[ch,i].P+5*fPeaks[ch,i].S)<fDataCounter
  then NextPeak(ch,i);
end;

procedure TGCStubInstrument.NextPeak(ch,n:integer);
begin
  with fPeaks[ch,n]
  do begin
    A:=random;
    P:=fNextPeakPos[ch];
    fNextPeakPos[ch]:=P+15*NExpoNoise;
    S:=1+3*random;
  end;
end;

function TGCStubInstrument.CalcPeaks(ch:integer;t:double):double;
var
  i:integer;
begin
  Result:=fBaseLines[ch];
  for i:=Low(fPeaks[ch]) to High(fPeaks[ch])
  do with fPeaks[ch,i]
  do Result:=Result+Gauss(A,t,P,S);
end;

function TGCStubInstrument.WorkThread:integer;

  procedure ReinitSetValues;
  var
    zt,zi,ot,oi:integer;
  begin
    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if fProgramsLength[zt,zi,ot,oi]>0
    then begin
      fValues[zt,zi,ot,oi,opSet]:=fPrograms[zt,zi,ot,oi,0].FinalVal;
    end;
  end;

var
  lasttime,time,dtime,ch:integer;
  intfreq,nstat:integer{dword};
  freq:single;
  i:integer;
  zt,zi,ot,oi:integer;

  progends,isfailure:boolean;
  timeinmin:double;
  databuf:array[0..999] of dword;
begin
  lasttime:=systicks();
  dtime:=0;
  nstat:=dsNormal;
  repeat
    Sleep(100);
    time:=systicks();
    inc(dtime,(time-lasttime));
    lasttime:=time;

    // Setting state

    if IsSetState
    then begin
      IsSetState:=false;
      if State<>NewState
      then begin
        State:=NewState;
        //StateChangeTime:=GetTickCount;
        ReinitSetValues;
        nstat:=NewState;
        DbgWrite('Software '+'changes state: '+sStates[nstat]);
        // compatibility with old things
        if nstat=dsStarting then CallNotify(rnStart)
        else if nstat<>dsPreRun then CallNotify(rnStop);
        fDataCounter:=0;
      end;
    end;

    // Setting MIN values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opMin in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opMin);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opMin]:=fNewValues[zt,zi,ot,oi,opMin];
    end;

    // Setting MAX values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opMax in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opMax);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opMax]:=fNewValues[zt,zi,ot,oi,opMax];
    end;

    // Setting SET values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opSet in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opSet);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opSet]:=fNewValues[zt,zi,ot,oi,opSet];
    end;

    // Setting DRIFT values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opDrift in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opDrift);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opDrift]:=fNewValues[zt,zi,ot,oi,opDrift];
    end;

    // Setting GAS values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opGas in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opGas);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opGas]:=fNewValues[zt,zi,ot,oi,opGas];
    end;

    // Setting LIGHT values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if opLight in fSetValues[zt,zi,ot,oi]
    then begin
      exclude(fSetValues[zt,zi,ot,oi], opLight);
      // as if we send it to instrument and then read from it.
      fValues[zt,zi,ot,oi,opLight]:=fNewValues[zt,zi,ot,oi,opLight];
    end;

    // Setting programs

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    if fSetPrograms[zt,zi,ot,oi]
    then begin
      fSetPrograms[zt,zi,ot,oi]:=false;
      // as if we send it to instrument and then read from it.
      for i:=0 to fNewProgramsLength[zt,zi,ot,oi]-1
      do fPrograms[zt,zi,ot,oi,i]:=fNewPrograms[zt,zi,ot,oi,i];
      fProgramsLength[zt,zi,ot,oi]:=fNewProgramsLength[zt,zi,ot,oi];
    end;

    // Checking hardware START button

    {$IFDEF MSWINDOWS}
    if ((GetAsyncKeyState(VK_LCONTROL) and GetAsyncKeyState(VK_RCONTROL))<>0)
    and not IsSetState
    then begin
      if (State<>dsStarting)
      then begin
        State:=dsStarting;
        //StateChangeTime:=GetTickCount;
        ReinitSetValues;
      end;
      nstat:=dsStarting;
      DbgWrite('Hardware START button detected');
      fDataCounter:=0;
      // compatibility with old things
      CallNotify(rnStart);
    end;
    {$ENDIF}

    // Processing programs
    if State=dsRun
    then begin
      progends:=true;
      isfailure:=false;
      time:=Self.StateTime;
      timeinmin:=time/60000;

      for zt:=Low(fValues) to High(fValues) do
      for zi:=Low(fValues[zt]) to High(fValues[zt]) do
      for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
      for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
      if fProgramsLength[zt,zi,ot,oi]>0
      then begin
        if GetProgramValue(fPrograms[zt,zi,ot,oi],fProgramsLength[zt,zi,ot,oi],timeinmin,fValues[zt,zi,ot,oi,opSet])
        then progends:=false;
        if (fValues[zt,zi,ot,oi,opSet]>fValues[zt,zi,ot,oi,opMax])
        or (fValues[zt,zi,ot,oi,opSet]<fValues[zt,zi,ot,oi,opMin])
        then isfailure:=true;
      end;
      if progends or isfailure
      then begin
        State:=dsPostRun;
        //StateChangeTime:=GetTickCount;
        ReinitSetValues;
        if isfailure
        then DbgWrite('Failure while processing programs, going to POSTRUN state')
        else DbgWrite('All programs ends, going to POSTRUN state');
        CallNotify(rnStop);
      end;
    end;

    {$IFDEF MSWINDOWS}
    // Processing "random" Exceptions

    if (State=dsPostRun) and (StateTime>3000) and (StateTime<3500) then PInteger(nil)^:=0;
    {$ENDIF}

    // Processing end of POSTRUN

    if (State=dsPostRun) and (StateTime>=60000)
    then begin
      State:=dsNormal;
      //StateChangeTime:=GetTickCount;
      DbgWrite('POSTRUN actions ends - going to NORMAL');
      //ReinitSetValues;
    end;

    // Retrieving CUR values

    for zt:=Low(fValues) to High(fValues) do
    for zi:=Low(fValues[zt]) to High(fValues[zt]) do
    for ot:=Low(fValues[zt,zi]) to High(fValues[zt,zi]) do
    for oi:=Low(fValues[zt,zi,ot]) to High(fValues[zt,zi,ot]) do
    begin
      // as if we read instrument current values.
      fValues[zt,zi,ot,oi,opCur]:=fValues[zt,zi,ot,oi,opCur]+0.01*(fValues[zt,zi,ot,oi,opSet]-fValues[zt,zi,ot,oi,opCur])+(Random-0.5)*0.02;
      // testing for overheating
      if (fValues[zt,zi,ot,oi,opCur]>fValues[zt,zi,ot,oi,opMax])
      or (fValues[zt,zi,ot,oi,opSet]>fValues[zt,zi,ot,oi,opMax])
      then begin
        fNewValues[zt,zi,ot,oi,opSet]:=fValues[zt,zi,ot,oi,opMin];
        include(fSetValues[zt,zi,ot,oi],opSet);
      end;
    end;

    // Retrieving hardware measuring data

    while (dtime>1000) do
    begin
      for ch:=0 to ChannelsSize-1 do
      if IsActiveChannel(ch) then
      begin
        freq:=Frequency[ch];
        intfreq:=round(freq);
        if fDataCounter=0 then ReInitPeaks(ch);
        //for i:=0 to intfreq-1 do databuf[i]:=round($1FFFFFFF*sin(2*pi*i/freq))+$3FFFFFFF+random($7FFFFF);
        for i:=0 to intfreq-1
        do begin
          {databuf[i]:=Round( $3FFFFFFF
            +$1FFFFFFF*(
              Gauss(0.01,i+fDataCounter*intfreq,16*intfreq,intfreq)+
              Gauss(0.05,i+fDataCounter*intfreq,32*intfreq,2*intfreq)+
              Gauss(0.2,i+fDataCounter*intfreq,48*intfreq,3*intfreq)+
              Gauss(0.8,i+fDataCounter*intfreq,64*intfreq,4*intfreq)+
              Gauss(0.1,i+fDataCounter*intfreq,80*intfreq,3*intfreq)+
              Gauss(0.5,i+fDataCounter*intfreq,96*intfreq,2*intfreq))
            +NGaussNoise*$7FFFF);}
          databuf[i]:=$3FFFFFFF+ Round($1FFFFFFF*CalcPeaks(ch,i/intfreq+fDataCounter)+NGaussNoise*$7FFFF);
        end;
        CallInt32Callback(ch,@databuf,intfreq,nstat);
        CheckPeaks(ch);
      end;
      fDataCounter:=fDataCounter+1;
      nstat:=dsNormal;
      dec(dtime,1000);
      CallNotify(rnAll);
    end;
  until NeedExit;
  result:=0;
end;

function TGCStubInstrument.SetFrequency(AChannel:integer;freq:single):single;
begin
  if freq>=0.5 then inherited SetFrequency(AChannel,round(freq));
  Result:=Frequency[AChannel];
end;

function TGCStubInstrument.GetChannelInfo(AChannel:integer;var DevInfo:TDeviceInfo):boolean;
begin
  Result:=false;
  if (AChannel<0) or (AChannel>=ChannelsSize) then exit;
  Result:=true;
  DevInfo.DevType:=0;
  DevInfo.FullScale:=$7FFFFFFF;
  DevInfo.DevMin:=-3.0;
  DevInfo.DevMax:=+3.0;
end;

function TGCStubInstrument.CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;
begin
  if AParamIndex=opName then Result:=false
  else
  Result:=(AZoneType>=Low(fValues))
       and(AZoneType<=High(fValues))
       and(AZoneIndex>=Low(fValues[AZoneType]))
       and(AZoneIndex<=High(fValues[AZoneType]))
       and(AObjectType>=Low(fValues[AZoneType,AZoneIndex]))
       and(AObjectType<=High(fValues[AZoneType,AZoneIndex]))
       and(AObjectIndex>=Low(fValues[AZoneType,AZoneIndex,AObjectType]))
       and(AObjectIndex<=High(fValues[AZoneType,AZoneIndex,AObjectType]))
       and(AParamIndex>=Low(fValues[AZoneType,AZoneIndex,AObjectType,AObjectIndex]))
       and(AParamIndex<=High(fValues[AZoneType,AZoneIndex,AObjectType,AObjectIndex]));
  {Result:=(AZoneType>=Low(fValues))
       and(AZoneType<=High(fValues))
       and(AZoneIndex>=Low(fValues[AZoneType]))
       and(AZoneIndex<=High(fValues[AZoneType]))
       and(AObjectType>=Low(fValues[AZoneType,AZoneIndex]))
       and(AObjectType<=High(fValues[AZoneType,AZoneIndex]))
       and(AObjectIndex>=Low(fValues[AZoneType,AZoneIndex,AObjectType]))
       and(AObjectIndex<=High(fValues[AZoneType,AZoneIndex,AObjectType]))
       and(AParamIndex>=opMax)
       and(AParamIndex<=opCur);}
end;

function TGCStubInstrument.CheckWriteZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):boolean;
begin
  Result:=(AParamIndex<>opCur)and CheckReadZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex);
  //DbgWrite(Format('CheckWrite %s%s,%s%s,%s: %s',[sZoneTypes[AZoneType],sCode(AZoneIndex),sObjTypes[AObjectType],sCode(AObjectIndex),sParamTypes[AParamIndex],sBool[Result]]));
end;

function TGCStubInstrument.SetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer;AValue:extended):boolean;
begin
  Result:=false;
  if (State=dsRun)or(IsSetState and (NewState=dsRun))
  then begin
    DbgWrite('Ignoring SetZoneValue while running');
    exit;
  end;
  fNewValues[AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex]:=AValue;
  //DbgWrite(Format('Set %s%s,%s%s,%s: %.2f',[sZoneTypes[AZoneType],sCode(AZoneIndex),sObjTypes[AObjectType],sCode(AObjectIndex),sParamTypes[AParamIndex],AValue]));
  include(fSetValues[AZoneType,AZoneIndex,AObjectType,AObjectIndex], AParamIndex);
  Result:=true;
end;

function TGCStubInstrument.GetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex:integer):extended;
begin
  Result:=fValues[AZoneType,AZoneIndex,AObjectType,AObjectIndex,AParamIndex];
end;

function TGCStubInstrument.SetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;
begin
  Result:=false;
  if (State=dsRun)or(IsSetState and (NewState=dsRun))
  then begin
    DbgWrite('Ignoring SetProgRamp while running');
    exit;
  end;
  //
  if (ARampIndex>=0)and assigned(PRec)
  then begin
    if ARampIndex>High(fPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex]) then exit;
    if (ARampIndex>=fProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex])
    or (not CompareMem(PRec,@fPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex],sizeof(TRamp)))
    then begin
      if (ARampIndex=0)
      then begin
        if PRec.ValRate<>0 then exit;
        SetZoneValue(AZoneType,AZoneIndex,AObjectType,AObjectIndex,opSet,PRec.FinalVal);
      end
      else begin
        //if PRec.ValRate=0 then exit;
      end;
      fNewPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex]:=PRec^;
      if (ARampIndex>=fProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex])
      then begin
        fNewProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]:=ARampIndex+1;
        DbgWrite('Ramp length set to: '+IntToStr(fNewProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]));
      end;
      fSetPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex]:=true;
    end;
  end
  else begin
    ARampIndex:=abs(ARampIndex);
    if ARampIndex<fProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]
    then begin
      fNewProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]:=ARampIndex;
      DbgWrite('Ramp length set to: '+IntToStr(fNewProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]));
      fSetPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex]:=true;
    end;
  end;
  Result:=true;
end;

function TGCStubInstrument.GetProgRamp(AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex:integer;PRec:PRamp):boolean;
begin
  Result:=false;
  if not assigned(PRec) then exit;
  if (ARampIndex<0)or(ARampIndex>=fProgramsLength[AZoneType,AZoneIndex,AObjectType,AObjectIndex]) then exit;
  PRec^:=fPrograms[AZoneType,AZoneIndex,AObjectType,AObjectIndex,ARampIndex];
  Result:=true;
end;

initialization
  DriverClass:=TGCStubDriver;
  InstrumentClass:=TGCStubInstrument;
end.
