///
/// File converter for AIA NetCDF chromatograpic data
/// should work MSVCPP and GCC compiled 

#include "stdlib.h"


#ifdef __GNUC__

#else
#include "stdafx.h"

#endif


#include "netcdf.h"

#include "time.h"

#include "stdio.h"
#include "math.h"

#ifdef __GNUC__
using namespace std;
#include "vrnt.h"
#include <string.h>
#include <wchar.h>

#define min fmin
#endif

//#include "../dcommon/specintf.h"
#include "../dcommon/ucintf.h"
#include "ncconv.h"




#pragma comment(lib,"netcdfs.lib")
//#pragma comment(lib,"uwin32.lib")


const int UnixDateDelta = 25569;
const int HoursPerDay   = 24;
const int MinsPerHour   = 60;
const int SecsPerMin    = 60;
const int MSecsPerSec   = 1000;
const int MinsPerDay    = HoursPerDay * MinsPerHour;
const int SecsPerDay    = MinsPerDay * SecsPerMin;
const int MSecsPerDay   = SecsPerDay * MSecsPerSec;


#ifndef __GNUC__
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif


int nc_check(int stat)
{
	if (stat)
	{
	printf("AIA filter status=%d: %s\r\n",stat,nc_strerror(stat));
	}
	return stat;
}

void varinit(VARIANT* v)
{
	memset(v,0,sizeof(*v));
}

double fval(VARIANT* v)
{
	switch(v->vt)
	{
		case VT_R4: return v->fltVal;
		case VT_R8: return v->dblVal;
		default: return -1;
	}
}


int charfromvar(VARIANT* v, char* buf, int* len)
{
	int wlen=0;
	if (v->vt!=VT_BSTR) return 0;
		
	if (v->bstrVal)
		wlen=wcslen(v->bstrVal);
	else
		return 0;
		
	
	#ifdef _WIN32
	WideCharToMultiByte(
					CP_ACP,            // code page
					WC_COMPOSITECHECK,            // performance and mapping flags
					v->bstrVal,    // wide-character string
					wlen,          // number of chars in string
					buf,  // buffer for new string
					*len, // size of buffer
					NULL, // default for unmappable chars
					NULL  // set when default char used
		);
		
	#endif

	*len = wlen;

	return(1);
}


static TUniChromEntryTable uc={0}; 

int NCCONV_API cnvInit(PUniChromEntryTable iuc)
{
	  if ((iuc!=NULL) && (iuc->dwVersion>=SPEC_API_VERSION) && (iuc->dwSize>=sizeof(*iuc)))
		{
			//uc = iuc*;
			memcpy(&uc,iuc,sizeof(uc));
			return(1);
		}
		else
			return 0;
}

int NCCONV_API cnvImport(int nFormatId, const PCHAR FileName,HSpec Asp)
{
	int ncid=-1,stat=0,ordid=-1,rtid=-1,sampintid=-1,detminid=-1,detmaxid=-1;
	int pointid=-1,peakid=-1, peaknid=-1,peakrtid=-1,peakstid=-1,peaketid=-1,
			peak_amount_id=-1,
			blstid=-1,bletid=-1,blsvid=-1,blevid=-1;
	size_t cpoints=0,cpeaks=0;
	float runtime=0,sampint=0,detmin=0,detmax=0;
	double amount=0;

	unsigned int starts=0,counts=0;
	VARIANT v={0};

	HPeak pk = HINVALID;

	size_t peak_name_len=0;
	
	tm reg_time={0};
	int tz_hour=0,tz_min=0;
	double datetime=0;

	float l,c,r,bs,bsv,be,bev;
	char buf[256];
	int bt,lp,cp,rp;

	unsigned int start[2] = {0};
	unsigned int count[2] = {0};

	void* pdata = NULL;

	char dsc[300]={0},templ[300]={0},detunit[300]={0},retunit[300]={0},separation[300]={0},
		dataset_timestamp[100]={0},
		injection_timestamp[100]={0};

	if (nFormatId!=1) return(0); 

#ifdef _WIN32
	__try
#endif
	{

	stat = nc_check(nc_open(FileName,NC_NOWRITE,&ncid));

	if (stat) return 0;

	stat = nc_get_att_text(ncid, NC_GLOBAL,"dataset_completeness",dsc);
  stat = nc_get_att_text(ncid, NC_GLOBAL,"aia_template_revision",templ);

  /*stat = nc_put_att_text(ncid, NC_GLOBAL,"netcdf_revision",3,"3.5");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"languages",12,"English only");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"administrative_comments",1," ");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_origin",8,"UniChrom");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_owner",8,"UniChrom");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"experiment_title",23,"SequenceLine: 2  Inj: 1");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"operator_name",2,"Me");*/

	stat = nc_get_att_text(ncid, NC_GLOBAL,"dataset_date_time_stamp",dataset_timestamp);
  stat = nc_get_att_text(ncid, NC_GLOBAL,"injection_date_time_stamp",injection_timestamp);


//	struct tm {
//       int tm_sec;     /* seconds after the minute - [0,59] */
//        int tm_min;     /* minutes after the hour - [0,59] */
//        int tm_hour;    /* hours since midnight - [0,23] */
//        int tm_mday;    /* day of the month - [1,31] */
//        int tm_mon;     /* months since January - [0,11] */
//        int tm_year;    /* years since 1900 */
//        int tm_wday;    /* days since Sunday - [0,6] */
//        int tm_yday;    /* days since January 1 - [0,365] */
//        int tm_isdst;   /* daylight savings time flag */
//        };
//
	const char dt_fmt[]="%4d%02d%02d%02d%02d%02d%3d%02d";


	if (strlen(dataset_timestamp)>5)
		sscanf(dataset_timestamp,
			dt_fmt,
			&reg_time.tm_year,
			&reg_time.tm_mon,
			&reg_time.tm_mday,
			&reg_time.tm_hour,
			&reg_time.tm_min,
			&reg_time.tm_sec,
			&tz_hour,&tz_min);
	
	if (strlen(injection_timestamp)>5)
		sscanf(injection_timestamp,
			dt_fmt,
			&reg_time.tm_year,
			&reg_time.tm_mon,
			&reg_time.tm_mday,
			&reg_time.tm_hour,
			&reg_time.tm_min,
			&reg_time.tm_sec,
			&tz_hour,&tz_min);

	
//	reg_time.tm_isdst = 1;
	reg_time.tm_year-=1900; // convert to year since 1900
	reg_time.tm_mon-=1; // mocrosoft counts months from 0

	tzset();
  // _timezone = - ((tz_hour*60)+tz_min)*60;
	time_t ltime = mktime(&reg_time) - ((tz_hour*60)+tz_min)*60 - 2*timezone; // timestamp in local timezone + timezone which have been written

//	_tzset();
	// local timestamp
	datetime = double(ltime)/double(SecsPerDay) + UnixDateDelta;// + double(tz_hour)/HoursPerDay + double(tz_min)/MinsPerDay; 


  stat = nc_get_att_text(ncid, NC_GLOBAL,"separation_experiment_type",separation);
  stat = nc_get_att_text(ncid, NC_GLOBAL,"detector_unit",detunit);
  stat = nc_get_att_text(ncid, NC_GLOBAL,"retention_unit",retunit);


//	printf("opening \"%s\" hSp=%X \n",FileName,Asp);


	stat = nc_inq_dimid(ncid, "point_number", &pointid);
	if (!stat) stat = nc_inq_dimlen(ncid,pointid, &cpoints);

	stat = nc_inq_dimid(ncid, "peak_number", &peakid);
	if (!stat) stat = nc_inq_dimlen(ncid,peakid, &cpeaks);

	if (cpeaks)
	{ 
		/*		stat = nc_def_var(ncid, "peak_name", NC_CHAR, 2,&idarr[0], &peaknid);
		stat = nc_def_var(ncid, "peak_retention_time", NC_FLOAT, 1,&peakid, &peaktid);
		stat = nc_def_var(ncid, "peak_start_time", NC_FLOAT, 1,&peakid, &peaksid);
		stat = nc_def_var(ncid, "peak_end_time", NC_FLOAT, 1,&peakid, &peakeid);
		stat = nc_def_var(ncid, "peak_width", NC_FLOAT, 1,&peakid, &peakwid);
		stat = nc_def_var(ncid, "peak_area", NC_FLOAT, 1,&peakid, &peakaid);
		stat = nc_def_var(ncid, "peak_area_percent", NC_FLOAT, 1,&peakid, &peakapid);
		stat = nc_def_var(ncid, "peak_height", NC_FLOAT, 1,&peakid, &peakhid);
		stat = nc_def_var(ncid, "peak_height_percent", NC_FLOAT, 1,&peakid, &peakhpid);
		stat = nc_def_var(ncid, "peak_asymmetry", NC_FLOAT, 1,&peakid, &peakasid);

		stat = nc_def_var(ncid, "baseline_start_time", NC_FLOAT, 1,&peakid, &bsstid);
		stat = nc_def_var(ncid, "baseline_start_value", NC_FLOAT, 1,&peakid, &bssvid);
		stat = nc_def_var(ncid, "baseline_stop_time", NC_FLOAT, 1,&peakid, &bsetid);
		stat = nc_def_var(ncid, "baseline_stop_value", NC_FLOAT, 1,&peakid, &bsevid);*/
		stat = nc_inq_varid(ncid, "peak_name", &peaknid);

		int dimids[2];	

		if (stat==NC_NOERR)
		{
			stat = nc_inq_vardimid(ncid, peaknid, dimids); // get the dimension ids of peakname var
			if (stat==0)
				stat = nc_inq_dimlen(ncid,dimids[1], &peak_name_len); // det the size of second dimension
		}

		if (peak_name_len==0) peak_name_len = 32;


		stat = nc_inq_varid(ncid, "peak_retention_time", &peakrtid);
		stat = nc_inq_varid(ncid, "peak_start_time", &peakstid);
		stat = nc_inq_varid(ncid, "peak_end_time", &peaketid);
		stat = nc_inq_varid(ncid, "peak_amount", &peak_amount_id);

		stat = nc_inq_varid(ncid, "baseline_start_time", &blstid);
		stat = nc_inq_varid(ncid, "baseline_start_value", &blsvid);
		stat = nc_inq_varid(ncid, "baseline_stop_time", &bletid);
		stat = nc_inq_varid(ncid, "baseline_stop_value", &blevid);

		
	}

	stat = nc_inq_varid(ncid, "ordinate_values", &ordid);


	pdata = malloc(cpoints*sizeof(float));

	sampint = -1;

	starts = 0;  // array of start points
	counts = cpoints; // array of number of points

	if (pdata)
	{
		stat = nc_inq_varid(ncid, "actual_run_time_length", &rtid);
		if (!stat) stat = nc_get_var_float(ncid, rtid, &runtime);
		if (stat)
				runtime = -1; // failure condition

		stat = nc_inq_varid(ncid, "actual_sampling_interval", &sampintid);
		if (!stat) stat = nc_get_var_float(ncid, sampintid, &sampint);
		if (stat)
				sampint = -1;

		stat = nc_inq_varid(ncid, "detector_minimum_value", &detminid);
		if (!stat) stat = nc_get_var_float(ncid, detminid, &detmin);

		stat = nc_inq_varid(ncid, "detector_maximum_value", &detmaxid);
		if (!stat) stat = nc_get_var_float(ncid, detmaxid, &detmax);

		if (detmin>=detmax) detmax=detmin+3;


		v.dblVal = datetime; // datetime of layer
		v.vt = VT_R8;
		uc.SetPropValue(Asp, 0,"LastChanged",v);

		
		varinit(&v);
		v.intVal=1;
		v.vt = VT_I2;
	
		if (uc.SetPropValue(Asp, -1,"Lock",v))
		{
			v.fltVal = 3;//detmax;
			v.vt = VT_R4;
			uc.SetPropValue(Asp, -1,"DevMax",v);

			v.fltVal = -3;//detmin;
			v.vt = VT_R4;
			uc.SetPropValue(Asp, -1,"DevMin",v);

			v.intVal = 6000;//0x00ffffffL;
			v.vt = VT_I4;
			uc.SetPropValue(Asp, -1,"FullScale",v);

			varinit(&v);
			v.intVal = 1;
			v.vt = VT_I2;
			uc.SetPropValue(Asp, -1,"XUnitType",v);
			
			v.fltVal = 0.0;
			v.vt = VT_R4;
			uc.SetPropValue(Asp, -1,"XStart",v);

			if (runtime<=0)
			{
				runtime = (float)cpoints;
				sampint = 1;
			}

			if (sampint<=0) 
				sampint = cpoints/runtime;

			v.fltVal = runtime - sampint;
			v.vt = VT_R4;
			uc.SetPropValue(Asp, -1,"XEnd",v);


			v.fltVal = 1/sampint;
			v.vt = VT_R4;
			uc.SetPropValue(Asp, -1,"Freq",v);

			v.intVal=0;
			v.vt = VT_I2;
			uc.SetPropValue(Asp, -1,"Lock",v);
		} else
		{
			free(pdata);
			return(0); 
		}


		stat = nc_get_vara_float(ncid,ordid,&starts,&counts,(float*)pdata);

		uc.SetDataLayer(Asp,1,cpoints,sizeof(float),pdata);
	}

	free(pdata);


	if (cpeaks) // any peaks
	{
	

		for (int i=0;i<int(cpeaks);i++)
		{
			start[0] = i; start[1] = 0;
			count[0] = 1; count[1] = peak_name_len;
			
			memset(buf,0,sizeof(buf));

			lp = -1;
			cp = -1;
			rp = -1;

			if (peaknid>=0) // get peak name
			{
				stat = nc_get_vara_text(ncid, peaknid, start, count, buf);

				if (stat==0)
				for (int i=min(sizeof(buf),peak_name_len)-1;i>=0;i--) // cleanup tailing spaces
				{
					if ((unsigned char)(buf[i])<=(unsigned char)(0x20)) 
						buf[i]=0x00;
					else
						break;
				}
				else
					buf[0]=0x00;

			}
			
			if (peakstid>=0) 
			{
				stat =  nc_get_vara_float(ncid, peakstid,&start[0],&count[0], &l);
				if (stat==0)
					lp = l/sampint+1;
			}

			if (peakrtid>=0)
			{
				stat = nc_get_vara_float(ncid, peakrtid,&start[0],&count[0], &c);
				if (stat==0)
					cp = c/sampint+1;
			}
			
			if (peaketid>=0)
			{
				stat =  nc_get_vara_float(ncid, peaketid,&start[0],&count[0], &r);
				if (stat==0)
					rp = r/sampint+1;
				else
					rp = -1;
			}

			if (peak_amount_id>=0)
			{
				stat =  nc_get_vara_float(ncid, peak_amount_id,&start[0],&count[0], &r);
				if (stat==0)
					amount = r;
				else
					amount = 0;
				
			}

			// bs - baseline start x
			// bsv - baseline start y value
			// be - baseline end x
			// bev - baseline end y value

			stat = nc_get_vara_float(ncid, blstid,&start[0],&count[0], &bs);
			stat = nc_get_vara_float(ncid, blsvid,&start[0],&count[0], &bsv);
			stat = nc_get_vara_float(ncid, bletid,&start[0],&count[0], &be);
			stat = nc_get_vara_float(ncid, blevid,&start[0],&count[0], &bev);

			if (lp<0)
				lp = (bs/sampint/*+1*/);

			if (rp<0)
				rp = (be/sampint/*1*/);


			pk = uc.AddPeak(Asp,-1,lp,cp,rp);

			if (pk)
			{

				double alpha = (bev-bsv)*sampint/(be-bs);
				double beta  = bsv-alpha*bs/sampint;

				bt = PAR_FLOAT;
				uc.SetPeakParam(pk, piAlpha, bt, &alpha);

				bt = PAR_FLOAT;
				uc.SetPeakParam(pk, piBeta, bt, &beta);

				bt = PAR_FLOAT;
				uc.SetPeakParam(pk, piConc, bt, &amount);

				bt = PAR_STR;
				uc.SetPeakParam(pk, piName, bt, &buf[0]);
			}

			//
		}
		
	}


		v.intVal=1;
		v.vt = VT_I2;

		uc.SetPropValue(Asp, -1,"Lock",v);

		v.intVal=0;
		v.vt = VT_I2;

		uc.SetPropValue(Asp, -1,"Lock",v);
	}
	
#ifdef _WIN32
	__finally
#endif
	{

	nc_close(ncid);
	}

	return 1;
}


int NCCONV_API cnvExport(int nFormatId, const PCHAR FileName,HSpec Asp)
{
	int ncid=0,stat,pointid,peakid,errid,ordid,
		dimid,
		peaknid,
		peaktid,peaksid,peakeid,peakwid,
		peakaid,peakapid,
		peakhid,peakhpid,
		peak_amount_id,
		peakasid,
		bsstid,bssvid,bsetid,bsevid,
		//xid,
		detmaxid,detminid,runtimeid,sampid,delayid,anyvar;
  size_t start,count;
  float detmax,detmin,devmin,devmax,runtime,sampling,f;
	int FullScale;
	double datetime=0;
	time_t ldatetime=0;

	void* pdata;

	VARIANT v;

	int nPoints = 0;
	int nxUnitType = 0;
	int nPeaks = 0;
	HPeak aPeak;
	int bt,ival,max_peak_name;
	char buf[1000];


	FullScale = 0x00ffffffL;
	detmax = 1e9;
  detmin = 0;


	nPeaks = uc.GetNPeaks(Asp,-1);

	// determine the real peak count w/o fictives
	for (int i=nPeaks-1;i>=0;i--)
	{
		aPeak = uc.GetPeak(Asp,-1,i);
		if (aPeak==NULL) break;
		if (uc.GetPeakParam(aPeak, piState, &bt, &ival) && bt==PAR_INT)
		{	
			// check for fictive peak
			if (ival & pmFictive) 
				nPeaks--;
			else 
				break;
				
		}
	}

	max_peak_name = 2;

	memset(buf,0,sizeof(buf));

	for (int j=0;j<nPeaks;j++)
	{
		aPeak = uc.GetPeak(Asp,-1,j);

		if (uc.GetPeakParam(aPeak, piName, &bt, &buf) && bt==PAR_STR)
		{
			int ll = strlen(buf);
			while (ll>max_peak_name) max_peak_name*=2;
			if (max_peak_name>255) 
			{
				max_peak_name = 255;
				break;
			}
		}
	}

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"xUnitType",&v))
		nxUnitType = v.intVal;

	if (uc.GetPropValue(Asp,-1,"NPoints",&v))
		nPoints = v.intVal;

	if (uc.GetPropValue(Asp,-1,"XEnd",&v))
		runtime = float(fval(&v));


	varinit(&v);

	if (uc.GetPropValue(Asp,0,"LastChanged",&v))
		datetime = v.dblVal;

	ldatetime = floor((datetime - UnixDateDelta) * SecsPerDay );

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"XStep",&v))
		sampling = float(fval(&v));

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"DevMin",&v))
		devmin = float(fval(&v));

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"DevMax",&v))
		devmax = float(fval(&v));

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"FullScale",&v))
	{
		FullScale = v.intVal / 100;
		//detmax  = FullScale;
	}

	// seconds is default

	if (nxUnitType==0) // minutes
	{
		runtime *=60;
		sampling *=60;
	}
	

	// do nothing for empty arrays
	if (nPoints==0) return 0;

#ifdef _WIN32
	__try 
#endif
	{
	
  stat = nc_check(nc_create(FileName,NC_WRITE,&ncid));

	if (stat) return 0;
  
// here goes data definition

// define dimensions

	//char dimfmt[50]={0};

	//sprintf(dimfmt,"_%d_byte_string", max_peak_name);

	stat = nc_def_dim(ncid, /*dimfmt*/"peak_name_len", max_peak_name, &dimid);

	int varid;
	stat = nc_def_dim(ncid, "_64_byte_string", 64, &varid);

	stat = nc_def_dim(ncid, "point_number", nPoints, &pointid);
  stat = nc_def_dim(ncid, "peak_number", nPeaks, &peakid);
  stat = nc_def_dim(ncid, "error_number", 0, &errid);
  

  /*

        float  ;
        float ordinate_values(point_number) ;
                ordinate_values:uniform_sampling_flag = "Y" ;
                ordinate_values:autosampler_position = "1.5" ;
        float peak_retention_time(peak_number) ;
        float peak_start_time(peak_number) ;
        float peak_end_time(peak_number) ;
        float peak_width(peak_number) ;
        float peak_area(peak_number) ;
        float peak_area_percent(peak_number) ;
        float peak_height(peak_number) ;
        float peak_height_percent(peak_number) ;
        float peak_asymmetry(peak_number) ;
        float baseline_start_time(peak_number) ;
        float baseline_start_value(peak_number) ;
        float baseline_stop_time(peak_number) ;
        float baseline_stop_value(peak_number) ;
        char peak_start_detection_code(peak_number, _2_byte_string) ;
        char peak_stop_detection_code(peak_number, _2_byte_string) ;
        float migration_time(peak_number) ;
        float peak_area_square_root(peak_number) ;
        short manually_reintegrated_peaks(peak_number) ;*/

	stat = nc_def_var(ncid, "detector_maximum_value", NC_FLOAT, 0,&anyvar, &detmaxid);
  stat = nc_def_var(ncid, "detector_minimum_value", NC_FLOAT, 0,&anyvar, &detminid);
  stat = nc_def_var(ncid, "actual_run_time_length", NC_FLOAT, 0,&anyvar, &runtimeid);
  stat = nc_def_var(ncid, "actual_delay_time", NC_FLOAT, 0,&anyvar, &delayid);
  stat = nc_def_var(ncid, "actual_sampling_interval", NC_FLOAT, 0,&anyvar, &sampid);

  stat = nc_def_var(ncid, "ordinate_values", NC_FLOAT, 1,&pointid, &ordid);

	if (nPeaks)
	{
		int idarr[2] = {peakid,dimid};
		stat = nc_def_var(ncid, "peak_name", NC_CHAR, 2,&idarr[0], &peaknid);
		stat = nc_def_var(ncid, "peak_retention_time", NC_FLOAT, 1,&peakid, &peaktid);
		stat = nc_def_var(ncid, "peak_start_time", NC_FLOAT, 1,&peakid, &peaksid);
		stat = nc_def_var(ncid, "peak_end_time", NC_FLOAT, 1,&peakid, &peakeid);
		stat = nc_def_var(ncid, "peak_width", NC_FLOAT, 1,&peakid, &peakwid);
		stat = nc_def_var(ncid, "peak_area", NC_FLOAT, 1,&peakid, &peakaid);
		stat = nc_def_var(ncid, "peak_area_percent", NC_FLOAT, 1,&peakid, &peakapid);
		stat = nc_def_var(ncid, "peak_height", NC_FLOAT, 1,&peakid, &peakhid);
		stat = nc_def_var(ncid, "peak_height_percent", NC_FLOAT, 1,&peakid, &peakhpid);
		stat = nc_def_var(ncid, "peak_amount", NC_FLOAT, 1,&peakid, &peak_amount_id);
		stat = nc_def_var(ncid, "peak_asymmetry", NC_FLOAT, 1,&peakid, &peakasid);

		stat = nc_def_var(ncid, "baseline_start_time", NC_FLOAT, 1,&peakid, &bsstid);
		stat = nc_def_var(ncid, "baseline_start_value", NC_FLOAT, 1,&peakid, &bssvid);
		stat = nc_def_var(ncid, "baseline_stop_time", NC_FLOAT, 1,&peakid, &bsetid);
		stat = nc_def_var(ncid, "baseline_stop_value", NC_FLOAT, 1,&peakid, &bsevid);
	}



  stat = nc_put_att_text(ncid, ordid,"uniform_sampling_flag",1,"Y");
  stat = nc_put_att_text(ncid, ordid,"autosampler_position",3,"1.1");

//  stat := nc_def_var(ncid, "raw_data_retention", NC_FLOAT, 1,pointid, xid);

/*                 :dataset_completeness = "C1+C2" ;
             :aia_template_revision = "1.0" ;
             :netcdf_revision = "2.3" ;
             :languages = "English only" ;
             :injection_date_time_stamp = "19000101000000+0000" ;
             :HP_injection_time = "8/28/95 2:33:12 PM" ;
             :experiment_title = "SequenceLine: 2  Inj: 1" ;
             :operator_name = "TJS & PL" ;
             :separation_experiment_type = "gas chromatography" ;
             :source_file_reference = "C:\\HPCHEM\\1\\DATA\\DEMO\\100FID50.D
             :sample_name = "Ultra Mix 200:1" ;
             :sample_id = "" ;
             :detector_unit = "pA" ;
             :detection_method_name = "100A50HZ.M" ;
             :detector_name = "FID1 A, " ;
             :retention_unit = "seconds" ;*/

  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_completeness",5,"C1+C2");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"aia_template_revision",3,"1.0");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"netcdf_revision",3,"3.5");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"languages",12,"English only");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"administrative_comments",1," ");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_origin",8,"UniChrom");
  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_owner",8,"UniChrom");

	tm* rt;

	rt = gmtime(&ldatetime);


	int cb = strftime(buf,sizeof(buf),"%Y%m%d%H%M%S",rt);
	
	char* stbuf = (char*)(buf+cb);

	sprintf(stbuf,"%+02d%02d",-timezone/3600,abs(timezone) % 3600);

	//_timezone/3600,0);

  stat = nc_put_att_text(ncid, NC_GLOBAL,"dataset_date_time_stamp",strlen(buf),buf);
  stat = nc_put_att_text(ncid, NC_GLOBAL,"injection_date_time_stamp",strlen(buf),buf);

	int mblen=0;

	varinit(&v);

	if (uc.GetPropValue(Asp,-1,"CurLayDesc",&v))
	{
		mblen = sizeof(buf);
		charfromvar(&v,buf,&mblen);
	} 
	else
	{
		strcpy(buf,"none");
		mblen = 4;
	}

	stat = nc_put_att_text(ncid, NC_GLOBAL,"experiment_title",mblen,buf);

  stat = nc_put_att_text(ncid, NC_GLOBAL,"operator_name",2,"Me");

	varinit(&v);

	if (!uc.GetPropValue(Asp,-1,"MethodType",&v))
		v.intVal=0;

	if (v.intVal==1)
			strcpy(buf,"liquid chromatography");
		else
			strcpy(buf,"gas chromatography");

  stat = nc_put_att_text(ncid, NC_GLOBAL,"separation_experiment_type",strlen(buf),buf);

	varinit(&v);
	if (uc.GetPropValue(Asp,-1,"UnitType",&v))
	{
		
	} else
		v.intVal = 1;

	switch (v.intVal)
	{
	case 0: strcpy(buf,"mV");break;
	case 1: strcpy(buf,"pA"); break;
	case 3: strcpy(buf,"AU");break;
	default: strcpy(buf,"cnt");
	}

  stat = nc_put_att_text(ncid, NC_GLOBAL,"detector_unit",strlen(buf),buf);


  stat = nc_put_att_text(ncid, NC_GLOBAL,"retention_unit",7,"seconds");



	// end definitions for dataset
  stat = nc_enddef(ncid);

  // put significant variables
	f = 0;
  stat = nc_put_var_float(ncid,detmaxid,&detmax);
  stat = nc_put_var_float(ncid,detminid,&detmin);
  stat = nc_put_var_float(ncid,runtimeid,&runtime);
  stat = nc_put_var_float(ncid,sampid,&sampling);
  stat = nc_put_var_float(ncid,delayid,&f);

	// begin data storage
  start = 0;
  count = nPoints;

	int nPointSize = 0;

	pdata = malloc(nPoints*sizeof(float));

	if (pdata)
	{
		if (uc.GetDataLayer(Asp,-1,&nPoints,&nPointSize,pdata))
		{
			/*float yconv = (devmax-devmin)/FullScale;
			float* pf=(float*)pdata;

			for (int i=0;i<nPoints;i++)
			{	
				*pf++ *= yconv;
			}*/

			stat = nc_check(nc_put_vara_float(ncid, ordid, &start, &count, (float*)pdata));
		}
		
		free(pdata);
		pdata = NULL;
	} 
	else
	{
		nc_close(ncid);
		return 0;
	}
  
  // begin peak storage 
  if (nPeaks) 
	{
		int ival;
		double dval;
		float fval,alpha,beta;
		float yconv = (devmax-devmin)/FullScale;

		for (int i=0;i<nPeaks;i++)
		{
			aPeak = uc.GetPeak(Asp,-1,i);
			if (aPeak==NULL) break;
			start = i;
			count = 1;
			/*
			stat = nc_def_var(ncid, "peak_retention_time", NC_FLOAT, 1,&peakid, &peaktid);
		stat = nc_def_var(ncid, "peak_start_time", NC_FLOAT, 1,&peakid, &peaksid);
		stat = nc_def_var(ncid, "peak_end_time", NC_FLOAT, 1,&peakid, &peakeid);
		stat = nc_def_var(ncid, "peak_width", NC_FLOAT, 1,&peakid, &peakwid);
		stat = nc_def_var(ncid, "peak_area", NC_FLOAT, 1,&peakid, &peakaid);
		stat = nc_def_var(ncid, "peak_height_percent", NC_FLOAT, 1,&peakid, &peakcid);
		stat = nc_def_var(ncid, "peak_asymmetry", NC_FLOAT, 1,&peakid, &peakasid);

		stat = nc_def_var(ncid, "baseline_start_time", NC_FLOAT, 1,&peakid, &bsstid);
		stat = nc_def_var(ncid, "baseline_start_value", NC_FLOAT, 1,&peakid, &bssvid);
		stat = nc_def_var(ncid, "baseline_stop_time", NC_FLOAT, 1,&peakid, &bsetid);
		stat = nc_def_var(ncid, "baseline_stop_value", NC_FLOAT, 1,&peakid, &bsevid);
			*/

			// get parameter of peak
			if (uc.GetPeakParam(aPeak, piState, &bt, &ival) && bt==PAR_INT)
			{	
				// check for fictive peak
				//printf("peak %d state=%d\r\n",i,ival);
				if (ival & pmFictive) break;
				
			}

			varinit(&v);

			memset(buf,0,256);

			if (uc.GetPeakParam(aPeak, piName, &bt, &buf) && bt==PAR_STR)
			{	
				// check for fictive peak
				//printf("peak %d state=%d\r\n",i,ival);
				//if (ival & pmFictive) break;				
				unsigned int counts[2] = {1,strlen(buf)};
				unsigned int starts[2]={i,0};
				stat = nc_check(nc_put_vara_text(ncid, peaknid, &starts[0], &counts[0], &buf[0]));
			}

			alpha = 0;
			beta = 0;

			if (uc.GetPeakParam(aPeak, piX, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				if (!nxUnitType) fval*=60;
				stat = nc_check(nc_put_vara_float(ncid, peaktid, &start, &count, &fval));
			}

			if (uc.GetPeakParam(aPeak, piAlpha, &bt, &dval) && bt==PAR_FLOAT)
			{	
				alpha = dval;
			}

			if (uc.GetPeakParam(aPeak, piBeta, &bt, &dval) && bt==PAR_FLOAT)
			{	
				beta = dval;
			}

			if (uc.GetPeakParam(aPeak, piXLeft, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				if (!nxUnitType) fval*=60;
				stat = nc_check(nc_put_vara_float(ncid, peaksid, &start, &count, &fval));
				stat = nc_check(nc_put_vara_float(ncid, bsstid, &start, &count, &fval));

				float yleft = alpha*fval/sampling+beta;
				stat = nc_check(nc_put_vara_float(ncid, bssvid, &start, &count, &yleft));
			}

			if (uc.GetPeakParam(aPeak, piXRight, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				if (!nxUnitType) fval*=60;
				stat = nc_check(nc_put_vara_float(ncid, peakeid, &start, &count, &fval));
				stat = nc_check(nc_put_vara_float(ncid, bsetid, &start, &count, &fval));

				float yright = alpha*fval/sampling+beta;
				stat = nc_check(nc_put_vara_float(ncid, bsevid, &start, &count, &yright));
			}

			// put peak half-width
			if (uc.GetPeakParam(aPeak, piHalfWidth, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				stat = nc_check(nc_put_vara_float(ncid, peakwid, &start, &count, &fval));
			}

			// put peak area
			if (uc.GetPeakParam(aPeak, piArea, &bt, &dval) && bt==PAR_FLOAT)  
			{	
				fval = dval;

				fval /=(yconv*1000*sampling);

				stat = nc_check(nc_put_vara_float(ncid, peakaid, &start, &count, &fval));
			}

			// put peak concentration
			if (uc.GetPeakParam(aPeak, piConc, &bt, &dval) && bt==PAR_FLOAT) 
			{	
				fval = dval;
				
				stat = nc_check(nc_put_vara_float(ncid, peakapid, &start, &count, &fval));

				stat = nc_check(nc_put_vara_float(ncid, peak_amount_id, &start, &count, &fval));

				//fval = 0;
				stat = nc_check(nc_put_vara_float(ncid, peakhpid, &start, &count, &fval));
			}

			if (uc.GetPeakParam(aPeak, piAmplitude, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				fval /=(yconv*1000);
				stat = nc_check(nc_put_vara_float(ncid, peakhid, &start, &count, &fval));
			}

			if (uc.GetPeakParam(aPeak, piAsymmetry, &bt, &dval) && bt==PAR_FLOAT)
			{	
				fval = dval;
				fval *=100;
				stat = nc_check(nc_put_vara_float(ncid, peakasid, &start, &count, &fval));
			}
		}
	}
	}
   
#ifdef _WIN32
	__finally
#endif
	{
		if (pdata) free(pdata);
			
		nc_check(stat);

		nc_close(ncid);
	}

	return 1;
}

const char pcFormatDesc[]="AIA Chromatography (*.cdf;*.nc)|*.cdf;*.nc";
 
int NCCONV_API cnvEnumFormats(int nFormatId, PCHAR FormatName,int* bufsize, int* flags)
{
	if (nFormatId==0)
		return 1; // number of formats supported

	if (nFormatId==1)
	{
			if (bufsize && flags && (*bufsize>int (strlen(pcFormatDesc))))
			{
				// copy string name of converter
				strncpy(FormatName,pcFormatDesc,strlen(pcFormatDesc)+1);
				*bufsize = strlen(pcFormatDesc)+1;
				*flags = 3; // both directions
				
				return 1;	
			}
	}
	return 0;
}


// end of nnconv.cpp
