// SampADC.cpp : Defines the entry point for the DLL application.
//

#include "windows.h"
#include "devtypes.h"
#include "sampadc.h"
#include "stdio.h"
#include "math.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return true;
}


const ChannelCount = 4; // number of instrument channels

static TDebugWriteStr gDebugFunction={0};
static void* gDebugClient={0};


typedef struct tag_TClientDesc 
{
	HANDLE hThread;
	int fExitFlag;
	TPageNotify Notify;
	void* Client;
	TClient Channels[ChannelCount];
} TClientDesc;


static TClientDesc Clients[256]={0};


void DbgWrite(const char* msg)
{
	if ((gDebugFunction)&&(gDebugClient))
		gDebugFunction(gDebugClient,(char*)msg);
}


BOOL SAMPADC_API drvOpenEx(TDebugWriteStr ADebugFunction,void* AClient)
{
	gDebugFunction = ADebugFunction;
	gDebugClient = AClient;

	DbgWrite("Driver Opened");

	return true;
}

BOOL SAMPADC_API drvClose(void)
{
	DbgWrite("Driver Freed");

	return true;
}



int __stdcall WorkActions(TClientDesc* AClient)
{
	int cycle=0;

	while (AClient->fExitFlag==0) // exit flag not set
	{
		
		int data[ChannelCount]; // adc data

		// draw the 4 sinusoidal channels with different periods
		// actual ADC handling code should go here

		data[0] = 0x7fffff+(int)0x7fffff*sin(0.1*cycle);
		data[1] = 0x7fffff+(int)0x7fffff*sin(0.2*cycle);
		data[2] = 0x7fffff+(int)0x7fffff*sin(0.3*cycle);
		data[3] = 0x7fffff+(int)0x7fffff*sin(0.4*cycle);


		// pass the data to UniChrom

		int cpoints = 1; // pass 1 sample point for each channel;

		for (int i=0;i<ChannelCount;i++) // across the channels
		{
			// test if the channel is opened
			if ((AClient->Channels[i].Action)&&(AClient->Channels[i].Client))
			
			AClient->Channels[i].Action( // call the action (TInt32Callback) - exactly as exported from driver (drvOpenInt32ChannelEx)
				AClient->Channels[i].Client, // for specified client (UniChrom method)
				(PDWordArray)&data[i], // passing the pointer to data
				cpoints, // with number of points in array
				dsNormal); // state dsNormal, external start - dsRun
		}

		Sleep(20); // stop flooding. make something usable here
		cycle++;

		// Example: How to notify UniChrom on measurement completion
		// UniChrom stops on measurement time completion (automatically)
		// or upon notification code (rnXXXX)
		if(cycle>2000)
		{
			if ((AClient->Notify)&&(AClient->Client))
				AClient->Notify(AClient->Client,rnStop,0);
			cycle = 0;
		}

	}
	return 0;
}

int SAMPADC_API drvOpenInstrument(int AProfile,TPageNotify AnAction,void* AClient)
{
	if (Clients[AProfile].Client==NULL)
	{
		Clients[AProfile].Client = AClient;
		Clients[AProfile].Notify = AnAction; // state notification callback
		unsigned long thid;
		HANDLE hThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)WorkActions,(void*)&Clients[AProfile],0,&thid);
		Clients[AProfile].hThread = hThread;

		return AProfile+1; // this would be instrument handle
	}
	else
		return 0; // instrument busy
}

BOOL SAMPADC_API drvCloseInstrument(IHandle HInstrument)
{
	int AProfile = int(HInstrument)-1;

	if (Clients[AProfile].Client)
	{
		HANDLE hThread = Clients[AProfile].hThread;

		DbgWrite("Terminating thread");

		Clients[AProfile].fExitFlag = 1; // indicate exit from thread
		if (WaitForSingleObject(hThread,2000)!=WAIT_OBJECT_0)
		{
			TerminateThread(hThread,2);
			DbgWrite("Terminating forcibly");
		}
		else
			DbgWrite("Terminating normally");

		CloseHandle(hThread);

		memset(&Clients[AProfile],0,sizeof(Clients[AProfile]));

		DbgWrite("Cleanup");
   

		return true; // close instrument OK
	}
	else
		return false; // instrument already closed
}

BOOL SAMPADC_API drvOpenInt32ChannelEx(IHandle HInstrument,TInt32Callback AnAction,void* AClient,int AChannel)
{
	int AProfile = int(HInstrument)-1;
	char buf[200];

	if (AProfile<0) return false;

	if (AChannel>=ChannelCount) return false;

	if (Clients[AProfile].hThread)
	{
		if (Clients[AProfile].Channels[AChannel].Client) // there is client 
		{
			sprintf(buf,"Channel #%d busy",AChannel+1);
			DbgWrite(buf); 	
			return false;
		}
		else
		{
			Clients[AProfile].Channels[AChannel].Action = AnAction;
			Clients[AProfile].Channels[AChannel].Client = AClient;

			sprintf(buf,"Channel #%d opened",AChannel+1);

			DbgWrite(buf); 	
			return true;
		}
	}
	else
		return false;
}

BOOL SAMPADC_API drvCloseChannelEx(IHandle HInstrument,int AChannel)
{
	int AProfile = int(HInstrument)-1;

	if (AProfile<0) return false;

	if (AChannel>=ChannelCount) return false;

	if (Clients[AProfile].hThread)
	{
		if (Clients[AProfile].Channels[AChannel].Client) // there is client 
		{
			Clients[AProfile].Channels[AChannel].Action = NULL;
			Clients[AProfile].Channels[AChannel].Client = NULL;
			
			char buf[200];

			sprintf(buf,"Channel #%d closed",AChannel+1);

			DbgWrite(buf); 	
			return true;
		}
	}
	
	return false;
}

BOOL SAMPADC_API drvGetChannelInfo(IHandle HInstrument,int AChannel,TDeviceInfo* DevInfo)
{
	if (AChannel>=ChannelCount) return false;

	DevInfo->DevType=0;
	DevInfo->FullScale=0xFFFFFF; // 24 bit
	DevInfo->DevMin=-3.0;
	DevInfo->DevMax=+3.0;
	return true;
}

float SAMPADC_API drvGetChannelFrequency(IHandle HInstrument,int AChannel)
{
	return 20.0;
}

float SAMPADC_API drvSetChannelFrequency(IHandle HInstrument,int AChannel,float freq)
{
	return 20.0;
}
